@@ -39,15 +39,6 @@
 GST_DEBUG_CATEGORY_STATIC (pixbufscale_debug);
 #define GST_CAT_DEFAULT pixbufscale_debug
 
-/* elementfactory information */
-static const GstElementDetails pixbufscale_details =
-GST_ELEMENT_DETAILS ("GdkPixbuf image scaler",
-    "Filter/Effect/Video",
-    "Resizes video",
-    "Jan Schmidt <thaytan@mad.scientist.com>\n"
-    "Wim Taymans <wim.taymans@chello.be>\n"
-    "Renato Filho <renato.filho@indt.org.br>");
-
 /* GstPixbufScale signals and args */
 enum
 {
@@ -96,10 +87,6 @@ gst_pixbufscale_method_get_type (void)
   return pixbufscale_method_type;
 }
 
-static void gst_pixbufscale_base_init (gpointer g_class);
-static void gst_pixbufscale_class_init (GstPixbufScaleClass * klass);
-static void gst_pixbufscale_init (GstPixbufScale * pixbufscale,
-    GstPixbufScaleClass * kclass);
 static void gst_pixbufscale_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_pixbufscale_get_property (GObject * object, guint prop_id,
@@ -129,13 +116,18 @@ gst_pixbufscale_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &pixbufscale_details);
+  gst_element_class_set_details_simple (element_class, "GdkPixbuf image scaler",
+      "Filter/Effect/Video", "Resizes video",
+      "Jan Schmidt <thaytan@mad.scientist.com>, "
+      "Wim Taymans <wim.taymans@chello.be>, "
+      "Renato Filho <renato.filho@indt.org.br>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_pixbufscale_src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_pixbufscale_sink_template));
 }
+
 static void
 gst_pixbufscale_class_init (GstPixbufScaleClass * klass)
 {
@@ -152,7 +144,7 @@ gst_pixbufscale_class_init (GstPixbufScaleClass * klass)
       ARG_METHOD,
       g_param_spec_enum ("method", "method", "method",
           GST_TYPE_PIXBUFSCALE_METHOD, GST_PIXBUFSCALE_BILINEAR,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   trans_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_pixbufscale_transform_caps);
