@@ -67,7 +67,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420") "; "
         GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_BGR "; "
         GST_VIDEO_CAPS_RGBx "; " GST_VIDEO_CAPS_xRGB "; "
-        GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_xBGR)
+        GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_xBGR "; "
+        GST_VIDEO_CAPS_GRAY8)
     );
 /* *INDENT-ON* */
 
@@ -151,8 +152,7 @@ gst_jpeg_dec_finalize (GObject * object)
 
   jpeg_destroy_decompress (&dec->cinfo);
 
-  if (dec->tempbuf)
-    gst_buffer_unref (dec->tempbuf);
+  g_object_unref (dec->adapter);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -189,7 +189,8 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
   g_object_class_install_property (gobject_class, PROP_IDCT_METHOD,
       g_param_spec_enum ("idct-method", "IDCT Method",
           "The IDCT algorithm to use", GST_TYPE_IDCT_METHOD,
-          JPEG_DEFAULT_IDCT_METHOD, G_PARAM_READWRITE));
+          JPEG_DEFAULT_IDCT_METHOD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_jpeg_dec_change_state);
@@ -201,20 +202,31 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
 static boolean
 gst_jpeg_dec_fill_input_buffer (j_decompress_ptr cinfo)
 {
-/*
-  struct GstJpegDecSourceMgr *src_mgr;
   GstJpegDec *dec;
+  guint av;
 
-  src_mgr = (struct GstJpegDecSourceMgr*) &cinfo->src;
-  dec = GST_JPEG_DEC (src_mgr->dec);
-*/
-  GST_DEBUG_OBJECT (CINFO_GET_JPEGDEC (cinfo), "fill_input_buffer");
-/*
-  g_return_val_if_fail (dec != NULL, TRUE);
+  dec = CINFO_GET_JPEGDEC (cinfo);
+  g_return_val_if_fail (dec != NULL, FALSE);
+
+  av = gst_adapter_available_fast (dec->adapter);
+  GST_DEBUG_OBJECT (dec, "fill_input_buffer: fast av=%u, remaining=%u", av,
+      dec->rem_img_len);
+
+  if (av == 0) {
+    GST_DEBUG_OBJECT (dec, "Out of data");
+    return FALSE;
+  }
+
+  if (dec->rem_img_len < av)
+    av = dec->rem_img_len;
+  dec->rem_img_len -= av;
+
+  g_free (dec->cur_buf);
+  dec->cur_buf = gst_adapter_take (dec->adapter, av);
+
+  cinfo->src->next_input_byte = dec->cur_buf;
+  cinfo->src->bytes_in_buffer = av;
 
-  src_mgr->pub.next_input_byte = GST_BUFFER_DATA (dec->tempbuf);
-  src_mgr->pub.bytes_in_buffer = GST_BUFFER_SIZE (dec->tempbuf);
-*/
   return TRUE;
 }
 
@@ -228,11 +240,29 @@ gst_jpeg_dec_init_source (j_decompress_ptr cinfo)
 static void
 gst_jpeg_dec_skip_input_data (j_decompress_ptr cinfo, glong num_bytes)
 {
-  GST_DEBUG_OBJECT (CINFO_GET_JPEGDEC (cinfo), "skip %ld bytes", num_bytes);
+  GstJpegDec *dec = CINFO_GET_JPEGDEC (cinfo);
+
+  GST_DEBUG_OBJECT (dec, "skip %ld bytes", num_bytes);
 
   if (num_bytes > 0 && cinfo->src->bytes_in_buffer >= num_bytes) {
     cinfo->src->next_input_byte += (size_t) num_bytes;
     cinfo->src->bytes_in_buffer -= (size_t) num_bytes;
+  } else if (num_bytes > 0) {
+    gint available;
+
+    num_bytes -= cinfo->src->bytes_in_buffer;
+    cinfo->src->next_input_byte += (size_t) cinfo->src->bytes_in_buffer;
+    cinfo->src->bytes_in_buffer = 0;
+
+    available = gst_adapter_available (dec->adapter);
+    if (available < num_bytes || available < dec->rem_img_len) {
+      GST_WARNING_OBJECT (dec, "Less bytes to skip than available in the "
+          "adapter or the remaining image length %ld < %d or %u",
+          num_bytes, available, dec->rem_img_len);
+    }
+    num_bytes = MIN (MIN (num_bytes, available), dec->rem_img_len);
+    gst_adapter_flush (dec->adapter, num_bytes);
+    dec->rem_img_len -= num_bytes;
   }
 }
 
@@ -316,89 +346,34 @@ gst_jpeg_dec_init (GstJpegDec * dec)
 
   /* init properties */
   dec->idct_method = JPEG_DEFAULT_IDCT_METHOD;
-}
 
-static inline gboolean
-is_jpeg_start_marker (const guint8 * data)
-{
-  return (data[0] == 0xff && data[1] == 0xd8);
-}
-
-static gboolean
-gst_jpeg_dec_find_jpeg_header (GstJpegDec * dec)
-{
-  const guint8 *data;
-  guint size;
-
-  data = GST_BUFFER_DATA (dec->tempbuf);
-  size = GST_BUFFER_SIZE (dec->tempbuf);
-
-  g_return_val_if_fail (size >= 2, FALSE);
-
-  while (!is_jpeg_start_marker (data) || data[2] != 0xff) {
-    const guint8 *marker;
-    GstBuffer *tmp;
-    guint off;
-
-    marker = memchr (data + 1, 0xff, size - 1 - 2);
-    if (marker == NULL) {
-      off = size - 1;           /* keep last byte */
-    } else {
-      off = marker - data;
-    }
-
-    tmp = gst_buffer_create_sub (dec->tempbuf, off, size - off);
-    gst_buffer_unref (dec->tempbuf);
-    dec->tempbuf = tmp;
-
-    data = GST_BUFFER_DATA (dec->tempbuf);
-    size = GST_BUFFER_SIZE (dec->tempbuf);
-
-    if (size < 2)
-      return FALSE;             /* wait for more data */
-  }
-
-  return TRUE;                  /* got header */
+  dec->adapter = gst_adapter_new ();
 }
 
 static gboolean
 gst_jpeg_dec_ensure_header (GstJpegDec * dec)
 {
-  g_return_val_if_fail (dec->tempbuf != NULL, FALSE);
+  gint av;
+  gint offset;
 
-check_header:
-
-  /* we need at least a start marker (0xff 0xd8)
-   *   and an end marker (0xff 0xd9) */
-  if (GST_BUFFER_SIZE (dec->tempbuf) <= 4) {
-    GST_DEBUG ("Not enough data");
-    return FALSE;               /* we need more data */
+  av = gst_adapter_available (dec->adapter);
+  /* we expect at least 4 bytes, first of which start marker */
+  offset = gst_adapter_masked_scan_uint32 (dec->adapter, 0xffffff00, 0xffd8ff00,
+      0, av);
+  if (G_UNLIKELY (offset < 0)) {
+    GST_DEBUG_OBJECT (dec, "No JPEG header in current buffer");
+    /* not found */
+    if (av > 4)
+      gst_adapter_flush (dec->adapter, av - 4);
+    return FALSE;
   }
 
-  if (!is_jpeg_start_marker (GST_BUFFER_DATA (dec->tempbuf))) {
-    GST_DEBUG ("Not a JPEG header, resyncing to header...");
-    if (!gst_jpeg_dec_find_jpeg_header (dec)) {
-      GST_DEBUG ("No JPEG header in current buffer");
-      return FALSE;             /* we need more data */
-    }
-    GST_DEBUG ("Found JPEG header");
-    goto check_header;          /* buffer might have changed */
-  }
+  GST_DEBUG_OBJECT (dec, "Found JPEG header");
+  gst_adapter_flush (dec->adapter, offset);
 
   return TRUE;
 }
 
-#if 0
-static gboolean
-gst_jpeg_dec_have_end_marker (GstJpegDec * dec)
-{
-  guint8 *data = GST_BUFFER_DATA (dec->tempbuf);
-  guint size = GST_BUFFER_SIZE (dec->tempbuf);
-
-  return (size > 2 && data && is_jpeg_end_marker (data + size - 2));
-}
-#endif
-
 static inline gboolean
 gst_jpeg_dec_parse_tag_has_entropy_segment (guint8 tag)
 {
@@ -407,103 +382,107 @@ gst_jpeg_dec_parse_tag_has_entropy_segment (guint8 tag)
   return FALSE;
 }
 
-/* returns image length in bytes if parsed 
+/* returns image length in bytes if parsed
  * successfully, otherwise 0 */
 static guint
 gst_jpeg_dec_parse_image_data (GstJpegDec * dec)
 {
-  guint8 *start, *data, *end;
   guint size;
   gboolean resync;
+  GstAdapter *adapter = dec->adapter;
+  gint offset, noffset;
 
-  size = GST_BUFFER_SIZE (dec->tempbuf);
-  start = GST_BUFFER_DATA (dec->tempbuf);
-  end = start + size;
-  data = start;
+  size = gst_adapter_available (adapter);
 
-  g_return_val_if_fail (is_jpeg_start_marker (data), 0);
+  /* we expect at least 4 bytes, first of which start marker */
+  if (gst_adapter_masked_scan_uint32 (adapter, 0xffff0000, 0xffd80000, 0, 4))
+    return 0;
 
   GST_DEBUG ("Parsing jpeg image data (%u bytes)", size);
 
   GST_DEBUG ("Parse state: offset=%d, resync=%d, entropy len=%d",
       dec->parse_offset, dec->parse_resync, dec->parse_entropy_len);
 
-  /* resume from state offset (also skips start marker) */
-  data += (dec->parse_offset ? dec->parse_offset : 2);
+  /* offset is 2 less than actual offset;
+   * - adapter needs at least 4 bytes for scanning,
+   * - start and end marker ensure at least that much
+   */
+  /* resume from state offset */
+  offset = dec->parse_offset;
 
   while (1) {
     guint frame_len;
+    guint32 value;
 
-    /* do we need to resync? */
-    resync = (*data != 0xff);
-    if (resync) {
-      GST_DEBUG ("Lost sync at 0x%08" G_GINT64_MODIFIER "x, resyncing",
-          (gint64) (data - start));
-      /* at the very least we expect 0xff 0xNN, thus end-1 */
-      while (*data != 0xff && data < end - 1)
-        ++data;
-      if (G_UNLIKELY (*data != 0xff)) {
-        GST_DEBUG ("at end of input and no next marker found, need more data");
-        goto need_more_data;
-      }
+    noffset =
+        gst_adapter_masked_scan_uint32_peek (adapter, 0x0000ff00, 0x0000ff00,
+        offset, size - offset, &value);
+    /* lost sync if 0xff marker not where expected */
+    if ((resync = (noffset != offset))) {
+      GST_DEBUG ("Lost sync at 0x%08x, resyncing", offset + 2);
     }
     /* may have marker, but could have been resyncng */
     resync = resync || dec->parse_resync;
     /* Skip over extra 0xff */
-    while (*data == 0xff && data < end)
-      ++data;
+    while ((noffset >= 0) && ((value & 0xff) == 0xff)) {
+      noffset++;
+      noffset =
+          gst_adapter_masked_scan_uint32_peek (adapter, 0x0000ff00, 0x0000ff00,
+          noffset, size - noffset, &value);
+    }
     /* enough bytes left for marker? (we need 0xNN after the 0xff) */
-    if (data >= end) {
+    if (noffset < 0) {
       GST_DEBUG ("at end of input and no EOI marker found, need more data");
-      goto need_more_data1;
+      goto need_more_data;
     }
 
-    if (*data == 0xd9) {
-      GST_DEBUG ("0x%08" G_GINT64_MODIFIER "x: EOI marker",
-          (gint64) (data - start));
+    /* now lock on the marker we found */
+    offset = noffset;
+    value = value & 0xff;
+    if (value == 0xd9) {
+      GST_DEBUG ("0x%08x: EOI marker", offset + 2);
       /* clear parse state */
       dec->parse_resync = FALSE;
       dec->parse_offset = 0;
-      return (data - start + 1);
+      return (offset + 4);
     }
 
-    if (*data >= 0xd0 && *data <= 0xd7)
+    if (value >= 0xd0 && value <= 0xd7)
       frame_len = 0;
-    else if (data >= end - 2)
-      goto need_more_data1;
-    else
-      frame_len = GST_READ_UINT16_BE (data + 1);
-    GST_DEBUG ("0x%08" G_GINT64_MODIFIER "x: tag %02x, frame_len=%u",
-        (gint64) (data - start - 1), *data, frame_len);
+    else {
+      /* peek tag and subsequent length */
+      if (offset + 2 + 4 > size)
+        goto need_more_data;
+      else
+        gst_adapter_masked_scan_uint32_peek (adapter, 0x0, 0x0, offset + 2, 4,
+            &frame_len);
+      frame_len = frame_len & 0xffff;
+    }
+    GST_DEBUG ("0x%08x: tag %02x, frame_len=%u", offset + 2, value, frame_len);
     /* the frame length includes the 2 bytes for the length; here we want at
-     * least 2 more bytes at the end for an end marker, thus end-2 */
-    if (data + 1 + frame_len >= end - 2) {
-      if (resync) {
-        GST_DEBUG ("not a valid sync (not enough data).");
-        /* Since *data != 0xff, the next iteration will go into resync again. */
-        continue;
-      }
-      /* theoretically we could have lost sync and not really need more
-       * data, but that's just tough luck and a broken image then */
-      GST_DEBUG ("at end of input and no EOI marker found, need more data");
-      goto need_more_data1;
+     * least 2 more bytes at the end for an end marker */
+    if (offset + 2 + 2 + frame_len + 2 > size) {
+      goto need_more_data;
     }
 
-    if (gst_jpeg_dec_parse_tag_has_entropy_segment (*data)) {
-      guint8 *d2 = data + 1 + frame_len;
+    if (gst_jpeg_dec_parse_tag_has_entropy_segment (value)) {
       guint eseglen = dec->parse_entropy_len;
 
-      GST_DEBUG ("0x%08" G_GINT64_MODIFIER "x: finding entropy segment length",
-          (gint64) (data - start - 1));
+      GST_DEBUG ("0x%08x: finding entropy segment length", offset + 2);
+      noffset = offset + 2 + frame_len + dec->parse_entropy_len;
       while (1) {
-        if (d2 + eseglen >= end - 1) {
+        noffset = gst_adapter_masked_scan_uint32_peek (adapter, 0x0000ff00,
+            0x0000ff00, noffset, size - noffset, &value);
+        if (noffset < 0) {
           /* need more data */
-          dec->parse_entropy_len = eseglen;
-          goto need_more_data1;
+          dec->parse_entropy_len = size - offset - 4 - frame_len - 2;
+          goto need_more_data;
         }
-        if (d2[eseglen] == 0xff && d2[eseglen + 1] != 0x00)
+        if ((value & 0xff) != 0x00) {
+          eseglen = noffset - offset - frame_len - 2;
           break;
-        ++eseglen;
+        }
+        noffset++;
       }
       dec->parse_entropy_len = 0;
       frame_len += eseglen;
@@ -513,28 +492,24 @@ gst_jpeg_dec_parse_image_data (GstJpegDec * dec)
     if (resync) {
       /* check if we will still be in sync if we interpret
        * this as a sync point and skip this frame */
-      if (data[2 + frame_len] != 0xff) {
+      noffset = offset + frame_len + 2;
+      noffset = gst_adapter_masked_scan_uint32 (adapter, 0x0000ff00, 0x0000ff00,
+          noffset, 4);
+      if (noffset < 0) {
         /* ignore and continue resyncing until we hit the end
          * of our data or find a sync point that looks okay */
         continue;
       }
-      GST_DEBUG ("found sync at %p", data - size);
+      GST_DEBUG ("found sync at 0x%x", offset + 2);
     }
 
-    data += 1 + frame_len;
+    offset += frame_len + 2;
   }
 
   /* EXITS */
 need_more_data:
   {
-    dec->parse_offset = data - start;
-    dec->parse_resync = resync;
-    return 0;
-  }
-need_more_data1:
-  {
-    /* step back one to point at marker */
-    dec->parse_offset = (data - start) - 1;
+    dec->parse_offset = offset;
     dec->parse_resync = resync;
     return 0;
   }
@@ -746,6 +721,42 @@ gst_jpeg_dec_ensure_buffers (GstJpegDec * dec, guint maxrowbytes)
   return TRUE;
 }
 
+static void
+gst_jpeg_dec_decode_grayscale (GstJpegDec * dec, guchar * base[1],
+    guint width, guint height, guint pstride, guint rstride)
+{
+  guchar *rows[16];
+  guchar **scanarray[1] = { rows };
+  gint i, j, k;
+  gint lines;
+
+  GST_DEBUG_OBJECT (dec, "indirect decoding of grayscale");
+
+  if (G_UNLIKELY (!gst_jpeg_dec_ensure_buffers (dec, GST_ROUND_UP_32 (width))))
+    return;
+
+  memcpy (rows, dec->idr_y, 16 * sizeof (gpointer));
+
+  i = 0;
+  while (i < height) {
+    lines = jpeg_read_raw_data (&dec->cinfo, scanarray, DCTSIZE);
+    if (G_LIKELY (lines > 0)) {
+      for (j = 0; (j < DCTSIZE) && (i < height); j++, i++) {
+        gint p;
+
+        p = 0;
+        for (k = 0; k < width; k++) {
+          base[0][p] = rows[j][k];
+          p += pstride;
+        }
+        base[0] += rstride;
+      }
+    } else {
+      GST_INFO_OBJECT (dec, "jpeg_read_raw_data() returned 0");
+    }
+  }
+}
+
 static void
 gst_jpeg_dec_decode_rgb (GstJpegDec * dec, guchar * base[3],
     guint width, guint height, guint pstride, guint rstride)
@@ -1092,6 +1103,16 @@ gst_jpeg_dec_negotiate (GstJpegDec * dec, gint width, gint height, gint clrspc)
     /* equal for all components */
     dec->stride = gst_video_format_get_row_stride (format, 0, width);
     dec->inc = gst_video_format_get_pixel_stride (format, 0);
+  } else if (dec->cinfo.jpeg_color_space == JCS_GRAYSCALE) {
+    /* TODO is anything else then 8bit supported in jpeg? */
+    format = GST_VIDEO_FORMAT_GRAY8;
+    caps = gst_video_format_new_caps (format, width, height,
+        dec->framerate_numerator, dec->framerate_denominator, 1, 1);
+    dec->outsize = gst_video_format_get_size (format, width, height);
+    dec->offset[0] =
+        gst_video_format_get_component_offset (format, 0, width, height);
+    dec->stride = gst_video_format_get_row_stride (format, 0, width);
+    dec->inc = gst_video_format_get_pixel_stride (format, 0);
   } else {
     /* go for plain and simple I420 */
     /* TODO other YUV cases ? */
@@ -1122,7 +1143,10 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   GstFlowReturn ret = GST_FLOW_OK;
   GstJpegDec *dec;
   GstBuffer *outbuf = NULL;
-  guchar *data, *outdata;
+#ifndef GST_DISABLE_GST_DEBUG
+  guchar *data;
+#endif
+  guchar *outdata;
   guchar *base[3], *last[3];
   guint img_len, outsize;
   gint width, height;
@@ -1141,17 +1165,13 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   if (GST_BUFFER_IS_DISCONT (buf)) {
     GST_DEBUG_OBJECT (dec, "buffer has DISCONT flag set");
     dec->discont = TRUE;
-    if (!dec->packetized && dec->tempbuf != NULL) {
+    if (!dec->packetized && gst_adapter_available (dec->adapter)) {
       GST_WARNING_OBJECT (dec, "DISCONT buffer in non-packetized mode, bad");
-      gst_buffer_replace (&dec->tempbuf, NULL);
+      gst_adapter_clear (dec->adapter);
     }
   }
 
-  if (dec->tempbuf) {
-    dec->tempbuf = gst_buffer_join (dec->tempbuf, buf);
-  } else {
-    dec->tempbuf = buf;
-  }
+  gst_adapter_push (dec->adapter, buf);
   buf = NULL;
 
   /* If we are non-packetized and know the total incoming size in bytes,
@@ -1159,10 +1179,10 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 
   if (!dec->packetized && (dec->segment.format == GST_FORMAT_BYTES) &&
       (dec->segment.stop != -1) &&
-      (GST_BUFFER_SIZE (dec->tempbuf) < dec->segment.stop)) {
+      (gst_adapter_available (dec->adapter) < dec->segment.stop)) {
     /* We assume that non-packetized input in bytes is *one* single jpeg image */
     GST_DEBUG ("Non-packetized mode. Got %d bytes, need %" G_GINT64_FORMAT,
-        GST_BUFFER_SIZE (dec->tempbuf), dec->segment.stop);
+        gst_adapter_available (dec->adapter), dec->segment.stop);
     goto need_more_data;
   }
 
@@ -1174,7 +1194,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
    * do some sanity checking instead of parsing all of 
    * the jpeg data */
   if (dec->packetized) {
-    img_len = GST_BUFFER_SIZE (dec->tempbuf);
+    img_len = gst_adapter_available (dec->adapter);
   } else {
     /* Parse jpeg image to handle jpeg input that
      * is not aligned to buffer boundaries */
@@ -1184,15 +1204,21 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
       goto need_more_data;
   }
 
+  dec->rem_img_len = img_len;
+
+  GST_LOG_OBJECT (dec, "image size = %u", img_len);
+
   /* QoS: if we're too late anyway, skip decoding */
   if (dec->packetized && !gst_jpeg_dec_do_qos (dec, timestamp))
     goto skip_decoding;
 
-  data = (guchar *) GST_BUFFER_DATA (dec->tempbuf);
-  GST_LOG_OBJECT (dec, "image size = %u", img_len);
+#ifndef GST_DISABLE_GST_DEBUG
+  data = (guint8 *) gst_adapter_peek (dec->adapter, 4);
+  GST_LOG_OBJECT (dec, "reading header %02x %02x %02x %02x", data[0], data[1],
+      data[2], data[3]);
+#endif
 
-  dec->jsrc.pub.next_input_byte = data;
-  dec->jsrc.pub.bytes_in_buffer = img_len;
+  gst_jpeg_dec_fill_input_buffer (&dec->cinfo);
 
   if (setjmp (dec->jerr.setjmp_buffer)) {
     code = dec->jerr.pub.msg_code;
@@ -1204,9 +1230,6 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     goto decode_error;
   }
 
-  GST_LOG_OBJECT (dec, "reading header %02x %02x %02x %02x", data[0], data[1],
-      data[2], data[3]);
-
   /* read header */
   hdr_ok = jpeg_read_header (&dec->cinfo, TRUE);
   if (G_UNLIKELY (hdr_ok != JPEG_HEADER_OK)) {
@@ -1258,11 +1281,13 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   /* sanity checks to get safe and reasonable output */
   switch (dec->cinfo.jpeg_color_space) {
     case JCS_GRAYSCALE:
+      if (dec->cinfo.num_components != 1)
+        goto invalid_yuvrgbgrayscale;
       break;
     case JCS_RGB:
       if (dec->cinfo.num_components != 3 || dec->cinfo.max_v_samp_factor > 1 ||
           dec->cinfo.max_h_samp_factor > 1)
-        goto invalid_yuvrgb;
+        goto invalid_yuvrgbgrayscale;
       break;
     case JCS_YCbCr:
       if (dec->cinfo.num_components != 3 ||
@@ -1270,7 +1295,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
           r_v < dec->cinfo.comp_info[1].v_samp_factor ||
           r_h < dec->cinfo.comp_info[0].h_samp_factor ||
           r_h < dec->cinfo.comp_info[1].h_samp_factor)
-        goto invalid_yuvrgb;
+        goto invalid_yuvrgbgrayscale;
       break;
     default:
       g_assert_not_reached ();
@@ -1322,6 +1347,10 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     base[1] = outdata + dec->offset[1];
     base[2] = outdata + dec->offset[2];
     gst_jpeg_dec_decode_rgb (dec, base, width, height, dec->inc, dec->stride);
+  } else if (dec->cinfo.jpeg_color_space == JCS_GRAYSCALE) {
+    base[0] = outdata + dec->offset[0];
+    gst_jpeg_dec_decode_grayscale (dec, base, width, height, dec->inc,
+        dec->stride);
   } else {
     /* mind the swap, jpeglib outputs blue chroma first
      * ensonic: I see no swap?
@@ -1401,16 +1430,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 
 skip_decoding:
 done:
-  if (GST_BUFFER_SIZE (dec->tempbuf) == img_len) {
-    gst_buffer_unref (dec->tempbuf);
-    dec->tempbuf = NULL;
-  } else {
-    GstBuffer *buf = gst_buffer_create_sub (dec->tempbuf, img_len,
-        GST_BUFFER_SIZE (dec->tempbuf) - img_len);
-
-    gst_buffer_unref (dec->tempbuf);
-    dec->tempbuf = buf;
-  }
+  gst_adapter_flush (dec->adapter, dec->rem_img_len);
 
 exit:
 
@@ -1467,7 +1487,8 @@ alloc_failed:
     GST_DEBUG_OBJECT (dec, "failed to alloc buffer, reason %s", reason);
     /* Reset for next time */
     jpeg_abort_decompress (&dec->cinfo);
-    if (GST_FLOW_IS_FATAL (ret)) {
+    if (ret != GST_FLOW_UNEXPECTED && ret != GST_FLOW_WRONG_STATE &&
+        ret != GST_FLOW_NOT_LINKED) {
       GST_ELEMENT_ERROR (dec, STREAM, DECODE,
           ("Buffer allocation failed, reason: %s", reason),
           ("Buffer allocation failed, reason: %s", reason));
@@ -1495,10 +1516,10 @@ unsupported_colorspace:
     ret = GST_FLOW_ERROR;
     goto done;
   }
-invalid_yuvrgb:
+invalid_yuvrgbgrayscale:
   {
     GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL),
-        ("Picture is corrupt or unhandled YUV/RGB layout"));
+        ("Picture is corrupt or unhandled YUV/RGB/grayscale layout"));
     ret = GST_FLOW_ERROR;
     goto done;
   }
@@ -1545,6 +1566,12 @@ gst_jpeg_dec_sink_event (GstPad * pad, GstEvent * event)
       GST_DEBUG_OBJECT (dec, "Aborting decompress");
       jpeg_abort_decompress (&dec->cinfo);
       gst_segment_init (&dec->segment, GST_FORMAT_UNDEFINED);
+      gst_adapter_clear (dec->adapter);
+      g_free (dec->cur_buf);
+      dec->cur_buf = NULL;
+      dec->parse_offset = 0;
+      dec->parse_entropy_len = 0;
+      dec->parse_resync = FALSE;
       gst_jpeg_dec_reset_qos (dec);
       break;
     case GST_EVENT_NEWSEGMENT:{
@@ -1633,8 +1660,9 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
       dec->next_ts = 0;
       dec->discont = TRUE;
       dec->parse_offset = 0;
+      dec->parse_entropy_len = 0;
       dec->parse_resync = FALSE;
-      dec->parse_entropy_len = FALSE;
+      dec->cur_buf = NULL;
       gst_segment_init (&dec->segment, GST_FORMAT_UNDEFINED);
       gst_jpeg_dec_reset_qos (dec);
     default:
@@ -1647,10 +1675,9 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      if (dec->tempbuf) {
-        gst_buffer_unref (dec->tempbuf);
-        dec->tempbuf = NULL;
-      }
+      gst_adapter_clear (dec->adapter);
+      g_free (dec->cur_buf);
+      dec->cur_buf = NULL;
       gst_jpeg_dec_free_buffers (dec);
       break;
     default:
