@@ -86,6 +86,7 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC_colr, "colr", 0,},
   {FOURCC_fiel, "fiel", 0,},
   {FOURCC_jp2x, "jp2x", 0,},
+  {FOURCC_alac, "alac", 0,},
   {FOURCC_wave, "wave", QT_FLAG_CONTAINER},
   {FOURCC_appl, "appl", QT_FLAG_CONTAINER},
   {FOURCC_esds, "esds", 0},
@@ -97,17 +98,20 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC__nam, "Name", QT_FLAG_CONTAINER,},
   {FOURCC_titl, "Title", QT_FLAG_CONTAINER,},
   {FOURCC__ART, "Artist", QT_FLAG_CONTAINER,},
+  {FOURCC_aART, "Album Artist", QT_FLAG_CONTAINER,},
   {FOURCC_auth, "Author", QT_FLAG_CONTAINER,},
   {FOURCC_perf, "Performer", QT_FLAG_CONTAINER,},
   {FOURCC__wrt, "Writer", QT_FLAG_CONTAINER,},
-  {FOURCC__grp, "Group", QT_FLAG_CONTAINER,},
+  {FOURCC__grp, "Grouping", QT_FLAG_CONTAINER,},
   {FOURCC__alb, "Album", QT_FLAG_CONTAINER,},
   {FOURCC_albm, "Album", QT_FLAG_CONTAINER,},
   {FOURCC__day, "Date", QT_FLAG_CONTAINER,},
   {FOURCC__cpy, "Copyright", QT_FLAG_CONTAINER,},
   {FOURCC__cmt, "Comment", QT_FLAG_CONTAINER,},
   {FOURCC__des, "Description", QT_FLAG_CONTAINER,},
+  {FOURCC_desc, "Description", QT_FLAG_CONTAINER,},
   {FOURCC_dscp, "Description", QT_FLAG_CONTAINER,},
+  {FOURCC__lyr, "Lyrics", QT_FLAG_CONTAINER,},
   {FOURCC__req, "Requirement", QT_FLAG_CONTAINER,},
   {FOURCC__enc, "Encoder", QT_FLAG_CONTAINER,},
   {FOURCC_gnre, "Genre", QT_FLAG_CONTAINER,},
@@ -115,12 +119,24 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC_disc, "Disc Number", QT_FLAG_CONTAINER,},
   {FOURCC_disk, "Disc Number", QT_FLAG_CONTAINER,},
   {FOURCC_cprt, "Copyright", QT_FLAG_CONTAINER,},
-  {FOURCC_cpil, "cpil", QT_FLAG_CONTAINER,},
+  {FOURCC_cpil, "Compilation", QT_FLAG_CONTAINER,},
+  {FOURCC_pgap, "Gapless", QT_FLAG_CONTAINER,},
+  {FOURCC_pcst, "Podcast", QT_FLAG_CONTAINER,},
   {FOURCC_tmpo, "Tempo", QT_FLAG_CONTAINER,},
   {FOURCC_covr, "Cover", QT_FLAG_CONTAINER,},
+  {FOURCC_sonm, "Sort Title", QT_FLAG_CONTAINER,},
+  {FOURCC_soal, "Sort Album", QT_FLAG_CONTAINER,},
+  {FOURCC_soar, "Sort Artist", QT_FLAG_CONTAINER,},
+  {FOURCC_soaa, "Sort Album Artist", QT_FLAG_CONTAINER,},
+  {FOURCC_soco, "Sort Composer", QT_FLAG_CONTAINER,},
+  {FOURCC_sosn, "Sort TV Show", QT_FLAG_CONTAINER,},
+  {FOURCC_tvsh, "TV Show", QT_FLAG_CONTAINER,},
+  {FOURCC_tven, "TV Episode ID", QT_FLAG_CONTAINER,},
+  {FOURCC_tvsn, "TV Season Number", QT_FLAG_CONTAINER,},
+  {FOURCC_tves, "TV Episode Number", QT_FLAG_CONTAINER,},
   {FOURCC_keyw, "Keywords", QT_FLAG_CONTAINER,},
   {FOURCC_kywd, "Keywords", QT_FLAG_CONTAINER,},
-  {FOURCC__too, "too", QT_FLAG_CONTAINER,},
+  {FOURCC__too, "Encoder", QT_FLAG_CONTAINER,},
   {FOURCC_____, "----", QT_FLAG_CONTAINER,},
   {FOURCC_data, "data", 0, qtdemux_dump_unknown},
   {FOURCC_free, "free", 0,},
