@@ -49,7 +49,7 @@ typedef enum
   GST_H263_PAYLOAD_PICTURE_FORMAT_PLUS = 7
 } GstRtpH263PayPictureFormat;
 
-static guint format_props[8][2] = { {254, 254},
+static const guint format_props[8][2] = { {254, 254},
 {6, 8},
 {9, 11},
 {18, 22},
@@ -64,7 +64,7 @@ static guint format_props[8][2] = { {254, 254},
  */
 #define MCBPC_I_LEN 9
 #define MCBPC_I_WID 6
-static guint32 mcbpc_I[9][6] = {
+static const guint32 mcbpc_I[9][6] = {
   {0x8000, 0x8000, 1, 0, 0, 3},
   {0x2000, 0xe000, 3, 0, 1, 3},
   {0x4000, 0xe000, 3, 1, 0, 3},
@@ -81,7 +81,7 @@ static guint32 mcbpc_I[9][6] = {
  */
 #define MCBPC_P_LEN 21
 #define MCBPC_P_WID 6
-static guint16 mcbpc_P[21][6] = {
+static const guint16 mcbpc_P[21][6] = {
   {0x8000, 0x8000, 1, 0, 0, 0},
   {0x3000, 0xf000, 4, 0, 1, 0},
   {0x2000, 0xf000, 4, 1, 0, 0},
@@ -110,7 +110,7 @@ static guint16 mcbpc_P[21][6] = {
  */
 #define CBPY_LEN 16
 #define CBPY_WID 7
-static guint8 cbpy_I[16][7] = {
+static const guint8 cbpy_I[16][7] = {
   {0x30, 0xf0, 4, 0, 0, 0, 0},
   {0x28, 0xf8, 5, 0, 0, 0, 1},
   {0x20, 0xf8, 5, 0, 0, 1, 0},
@@ -132,7 +132,7 @@ static guint8 cbpy_I[16][7] = {
 /*
  * P-frame CBPY (code, mask, nbits, Y0, Y1, Y2, Y3)
  */
-static guint8 cbpy_P[16][7] = {
+static const guint8 cbpy_P[16][7] = {
   {0x30, 0xf0, 4, 1, 1, 1, 1},
   {0x28, 0xf8, 5, 1, 1, 1, 0},
   {0x20, 0xf8, 5, 1, 1, 0, 1},
@@ -156,7 +156,7 @@ static guint8 cbpy_P[16][7] = {
  */
 #define TCOEF_LEN 103
 #define TCOEF_WID 6
-static guint16 tcoef[103][6] = {
+static const guint16 tcoef[103][6] = {
   {0x8000, 0xc000, 3, 0, 0, 1},
   {0xf000, 0xf000, 5, 0, 0, 2},
   {0x5400, 0xfc00, 7, 0, 0, 3},
@@ -267,7 +267,7 @@ static guint16 tcoef[103][6] = {
  */
 #define MVD_LEN 64
 #define MVD_WID 5
-static guint16 mvd[64][5] = {
+static const guint16 mvd[64][5] = {
   {0x0028, 0xfff8, 13, 0x0060, 0x0020},
   {0x0038, 0xfff8, 13, 0x0061, 0x0021},
   {0x0050, 0xfff0, 12, 0x0062, 0x0022},
@@ -434,7 +434,7 @@ gst_rtp_h263_pay_class_init (GstRtpH263PayClass * klass)
       PROP_MODE_A_ONLY, g_param_spec_boolean ("modea-only",
           "Fragment packets in mode A Only",
           "Disable packetization modes B and C", DEFAULT_MODE_A,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   GST_DEBUG_CATEGORY_INIT (rtph263pay_debug, "rtph263pay", 0,
       "H263 RTP Payloader");
@@ -785,7 +785,7 @@ gst_rtp_h263_pay_decode_mcbpc_P (guint32 value)
  * decode CBPY and return index in table or -1 if not found
  */
 static gint
-gst_rtp_h263_pay_decode_cbpy (guint32 value, guint8 cbpy_table[16][7])
+gst_rtp_h263_pay_decode_cbpy (guint32 value, const guint8 cbpy_table[16][7])
 {
 
   gint i;
@@ -1302,7 +1302,7 @@ gst_rtp_h263_pay_A_fragment_push (GstRtpH263Pay * rtph263pay,
       (context->gobs[last]->end - context->gobs[first]->start) + 1;
   pack->marker = FALSE;
 
-  if (last == format_props[context->piclayer->ptype_srcformat][0] - 1) {
+  if (last == context->no_gobs - 1) {
     pack->marker = TRUE;
   }
 
@@ -1645,6 +1645,7 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
     GstRtpH263PayBoundry bound;
     gint first;
     guint payload_len;
+    gboolean forcea = FALSE;
 
     GST_DEBUG ("Frame too large for MTU");
     /*
@@ -1660,10 +1661,16 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
     for (i = 0; i < format_props[context->piclayer->ptype_srcformat][0]; i++) {
       GST_DEBUG ("Searching for gob %d", i);
       if (!gst_rtp_h263_pay_gobfinder (rtph263pay, &bound)) {
-        GST_WARNING
-            ("No GOB's were found in data stream! Please enable RTP mode in encoder. Forcing mode A for now.");
-        ret = gst_rtp_h263_send_entire_frame (rtph263pay, context);
-        goto end;
+        if (i <= 1) {
+          GST_WARNING
+              ("No GOB's were found in data stream! Please enable RTP mode in encoder. Forcing mode A for now.");
+          ret = gst_rtp_h263_send_entire_frame (rtph263pay, context);
+          goto end;
+        } else {
+          /* try to send fragments corresponding to found GOBs */
+          forcea = TRUE;
+          break;
+        }
       }
 
       context->gobs[i] = gst_rtp_h263_pay_gob_new (&bound, i);
@@ -1673,6 +1680,10 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
           context->gobs[i]->start, context->gobs[i]->length,
           context->gobs[i]->ebit, context->gobs[i]->sbit);
     }
+    /* NOTE some places may still assume this to be the max possible */
+    context->no_gobs = i;
+    GST_DEBUG ("Found %d GOBS of maximum %d",
+        context->no_gobs, format_props[context->piclayer->ptype_srcformat][0]);
 
     // Make packages smaller than MTU
     //   A mode
@@ -1682,13 +1693,13 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
     first = 0;
     payload_len = 0;
     i = 0;
-    while (i < format_props[context->piclayer->ptype_srcformat][0]) {
+    while (i < context->no_gobs) {
 
       if (context->gobs[i]->length >= context->mtu) {
         if (payload_len == 0) {
 
           GST_DEBUG ("GOB len > MTU");
-          if (rtph263pay->prop_payload_mode) {
+          if (rtph263pay->prop_payload_mode || forcea) {
             payload_len = context->gobs[i]->length;
             goto force_a;
           }
@@ -1723,7 +1734,7 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
         GST_DEBUG ("GOB %d fills mtu", i);
         payload_len += context->gobs[i]->length;
         i++;
-        if (i == format_props[context->piclayer->ptype_srcformat][0]) {
+        if (i == context->no_gobs) {
           GST_DEBUG ("LAST GOB %d", i);
           goto payload_a_push;
         }
