@@ -45,13 +45,6 @@ enum
 /* 3 zero bytes syncword */
 static const guint8 sync_bytes[] = { 0, 0, 0, 1 };
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_h264depay_details =
-GST_ELEMENT_DETAILS ("RTP H264 depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts H264 video from RTP packets (RFC 3984)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_h264_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -113,7 +106,10 @@ gst_rtp_h264_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_h264_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_h264depay_details);
+  gst_element_class_set_details_simple (element_class, "RTP H264 depayloader",
+      "Codec/Depayloader/Network",
+      "Extracts H264 video from RTP packets (RFC 3984)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -292,9 +288,8 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     guint len, num_sps, num_pps;
     gint i;
     guint8 *data;
-    guint32 profile_id;
 
-    if (ps == NULL || profile == NULL)
+    if (ps == NULL)
       goto incomplete_caps;
 
     params = g_strsplit (ps, ",", 0);
@@ -334,16 +329,31 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     }
     g_strfreev (params);
 
+    if (num_sps == 0 || (GST_READ_UINT16_BE (sps[0]) < 3) || num_pps == 0) {
+      g_strfreev ((gchar **) pps);
+      g_strfreev ((gchar **) sps);
+      goto incomplete_caps;
+    }
+
     codec_data = gst_buffer_new_and_alloc (len);
     data = GST_BUFFER_DATA (codec_data);
 
     /* 8 bits version == 1 */
     *data++ = 1;
-    /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
-    sscanf (profile, "%6x", &profile_id);
-    *data++ = (profile_id >> 16) & 0xff;
-    *data++ = (profile_id >> 8) & 0xff;
-    *data++ = profile_id & 0xff;
+    if (profile) {
+      guint32 profile_id;
+
+      /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
+      sscanf (profile, "%6x", &profile_id);
+      *data++ = (profile_id >> 16) & 0xff;
+      *data++ = (profile_id >> 8) & 0xff;
+      *data++ = profile_id & 0xff;
+    } else {
+      /* extract from SPS */
+      *data++ = sps[0][3];
+      *data++ = sps[0][4];
+      *data++ = sps[0][5];
+    }
     /* 6 bits reserved | 2 bits lengthSizeMinusOn */
     *data++ = 0xff;
     /* 3 bits reserved | 5 bits numOfSequenceParameterSets */
@@ -389,66 +399,116 @@ incomplete_caps:
   }
 }
 
-static GstBuffer *
-gst_rtp_h264_depay_push_nal (GstRtpH264Depay * rtph264depay, GstBuffer * nal,
-    GstClockTime timestamp)
+/* SPS/PPS/IDR considered key, all others DELTA;
+ * so downstream waiting for keyframe can pick up at SPS/PPS/IDR */
+#define NAL_TYPE_IS_KEY(nt) (((nt) == 5) || ((nt) == 7) || ((nt) == 8))
+
+static gboolean
+gst_rtp_h264_depay_handle_nal (GstRtpH264Depay * rtph264depay, GstBuffer * nal,
+    GstClockTime in_timestamp)
 {
+  GstBaseRTPDepayload *depayload = GST_BASE_RTP_DEPAYLOAD (rtph264depay);
   gint nal_type;
+  guint size;
   guint8 *data;
   GstBuffer *outbuf = NULL;
-  guint outsize;
-  gboolean start = FALSE;
+  GstClockTime out_timestamp;
+  gboolean keyframe, out_keyframe;
+
+  size = GST_BUFFER_SIZE (nal);
+  if (G_UNLIKELY (size < 5))
+    goto short_nal;
 
   data = GST_BUFFER_DATA (nal);
 
-  /* caller ensures number of bytes available */
   nal_type = data[4] & 0x1f;
-  GST_DEBUG_OBJECT (rtph264depay, "collecting NALU type %d", nal_type);
-  /* consider a coded slices (IDR or not) to start a picture,
-   * (so ending the previous one) if first_mb_in_slice == 0
-   * (non-0 is part of previous one) */
-  /* NOTE this is not entirely according to Access Unit specs in 7.4.1.2.4,
-   * but in practice it works in sane cases, needs not much parsing,
-   * and also works with broken frame_num in NAL (where spec-wise would fail) */
-  if (nal_type == 1 || nal_type == 2 || nal_type == 5) {
-    if (rtph264depay->picture_start) {
+  GST_DEBUG_OBJECT (rtph264depay, "handle NAL type %d", nal_type);
+
+  keyframe = NAL_TYPE_IS_KEY (nal_type);
+
+  out_keyframe = keyframe;
+  out_timestamp = in_timestamp;
+
+  if (rtph264depay->merge) {
+    gboolean start = FALSE, complete = FALSE;
+
+    /* consider a coded slices (IDR or not) to start a picture,
+     * (so ending the previous one) if first_mb_in_slice == 0
+     * (non-0 is part of previous one) */
+    /* NOTE this is not entirely according to Access Unit specs in 7.4.1.2.4,
+     * but in practice it works in sane cases, needs not much parsing,
+     * and also works with broken frame_num in NAL (where spec-wise would fail) */
+    if (nal_type == 1 || nal_type == 2 || nal_type == 5) {
+      /* we have a picture start */
+      start = TRUE;
       if (data[5] & 0x80) {
-        /* first_mb_in_slice == 0 */
-        rtph264depay->picture_complete = TRUE;
-        start = TRUE;
+        /* first_mb_in_slice == 0 completes a picture */
+        complete = TRUE;
       }
-      /* otherwise simply collect */
-    } else {
-      rtph264depay->picture_start = TRUE;
+    } else if (nal_type >= 6 && nal_type <= 9) {
+      /* SEI, SPS, PPS, AU terminate picture */
+      complete = TRUE;
     }
-  } else if (nal_type >= 6 && nal_type <= 9) {
-    /* SEI, SPS, PPS, AU terminate picture */
-    if (rtph264depay->picture_start)
-      rtph264depay->picture_complete = TRUE;
-    /* otherwise simply collect */
+    GST_DEBUG_OBJECT (depayload, "start %d, complete %d", start, complete);
+
+    if (complete && rtph264depay->picture_start) {
+      guint outsize;
+
+      /* we had a picture in the adapter and we completed it */
+      GST_DEBUG_OBJECT (depayload, "taking completed AU");
+      outsize = gst_adapter_available (rtph264depay->picture_adapter);
+      outbuf = gst_adapter_take_buffer (rtph264depay->picture_adapter, outsize);
+
+      out_timestamp = rtph264depay->last_ts;
+      out_keyframe = rtph264depay->last_keyframe;
+
+      rtph264depay->last_keyframe = FALSE;
+      rtph264depay->picture_start = FALSE;
+    }
+
+    /* add to adapter */
+    GST_DEBUG_OBJECT (depayload, "adding NAL to picture adapter");
+    gst_adapter_push (rtph264depay->picture_adapter, nal);
+    rtph264depay->last_ts = in_timestamp;
+    rtph264depay->last_keyframe |= keyframe;
+    rtph264depay->picture_start |= start;
+  } else {
+    /* no merge, output is input nal */
+    GST_DEBUG_OBJECT (depayload, "using NAL as output");
+    outbuf = nal;
   }
 
-  if (rtph264depay->picture_complete) {
-    outsize = gst_adapter_available (rtph264depay->picture_adapter);
-    outbuf = gst_adapter_take_buffer (rtph264depay->picture_adapter, outsize);
-    rtph264depay->picture_complete = FALSE;
-    rtph264depay->picture_start = start;
-
-    if (G_UNLIKELY (rtph264depay->codec_data)) {
+  if (outbuf) {
+    /* prepend codec_data */
+    if (rtph264depay->codec_data) {
+      GST_DEBUG_OBJECT (depayload, "prepending codec_data");
       outbuf = gst_buffer_join (rtph264depay->codec_data, outbuf);
       rtph264depay->codec_data = NULL;
+      out_keyframe = TRUE;
     }
+    outbuf = gst_buffer_make_metadata_writable (outbuf);
 
-    GST_BUFFER_TIMESTAMP (outbuf) = rtph264depay->last_ts;
-    if (GST_CLOCK_TIME_IS_VALID (timestamp) &&
-        GST_CLOCK_TIME_IS_VALID (rtph264depay->last_ts))
-      GST_BUFFER_DURATION (outbuf) = timestamp - rtph264depay->last_ts;
+    GST_BUFFER_TIMESTAMP (outbuf) = out_timestamp;
+
+    if (out_keyframe)
+      GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
+    else
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
+
+    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+
+    gst_base_rtp_depayload_push (depayload, outbuf);
   }
 
-  rtph264depay->last_ts = timestamp;
-  gst_adapter_push (rtph264depay->picture_adapter, nal);
+  return TRUE;
 
-  return outbuf;
+  /* ERRORS */
+short_nal:
+  {
+    GST_WARNING_OBJECT (depayload, "dropping short NAL");
+    gst_buffer_unref (nal);
+    return FALSE;
+  }
 }
 
 static GstBuffer *
@@ -473,17 +533,18 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     guint8 nal_ref_idc;
     guint8 *outdata;
     guint outsize, nalu_size;
-    GstClockTime ts;
+    GstClockTime timestamp;
 
-    /* baseclass already did this, would be nice not to have to redo ... */
-    ts = GST_BUFFER_TIMESTAMP (buf);
-    ts = gst_segment_to_running_time (&depayload->segment, GST_FORMAT_TIME, ts);
+    timestamp = GST_BUFFER_TIMESTAMP (buf);
 
     payload_len = gst_rtp_buffer_get_payload_len (buf);
     payload = gst_rtp_buffer_get_payload (buf);
 
     GST_DEBUG_OBJECT (rtph264depay, "receiving %d bytes", payload_len);
 
+    if (payload_len == 0)
+      return NULL;
+
     /* +---------------+
      * |0|1|2|3|4|5|6|7|
      * +-+-+-+-+-+-+-+-+
@@ -520,11 +581,6 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
         rtph264depay->wait_start = FALSE;
 
-        /* prepend codec_data */
-        if (!rtph264depay->merge && rtph264depay->codec_data) {
-          gst_adapter_push (rtph264depay->adapter, rtph264depay->codec_data);
-          rtph264depay->codec_data = NULL;
-        }
 
         /* STAP-A    Single-time aggregation packet     5.7.1 */
         while (payload_len > 2) {
@@ -557,26 +613,17 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
           outdata += sizeof (sync_bytes);
           memcpy (outdata, payload, nalu_size);
 
-          if (!rtph264depay->merge)
-            gst_adapter_push (rtph264depay->adapter, outbuf);
-          else {
-            outbuf = gst_rtp_h264_depay_push_nal (rtph264depay, outbuf, ts);
-            if (outbuf) {
-              gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-              gst_base_rtp_depayload_push_ts (depayload, ts, outbuf);
-            }
-          }
+          gst_adapter_push (rtph264depay->adapter, outbuf);
 
           payload += nalu_size;
           payload_len -= nalu_size;
         }
 
-        if (!rtph264depay->merge) {
-          outsize = gst_adapter_available (rtph264depay->adapter);
-          outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
-          gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-          return outbuf;
-        }
+        outsize = gst_adapter_available (rtph264depay->adapter);
+        outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
+
+        gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf, timestamp);
+        break;
       }
       case 26:
         /* MTAP16    Multi-time aggregation packet      5.7.2 */
@@ -669,23 +716,7 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
             outdata[2] = (outsize >> 8);
             outdata[3] = (outsize);
           }
-
-          /* push codec_data first */
-          if (!rtph264depay->merge && rtph264depay->codec_data) {
-            gst_buffer_set_caps (rtph264depay->codec_data,
-                GST_PAD_CAPS (depayload->srcpad));
-            gst_base_rtp_depayload_push (depayload, rtph264depay->codec_data);
-            rtph264depay->codec_data = NULL;
-          }
-
-          if (rtph264depay->merge) {
-            outbuf = gst_rtp_h264_depay_push_nal (rtph264depay, outbuf, ts);
-            if (!outbuf)
-              break;
-          }
-
-          gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-          return outbuf;
+          gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf, timestamp);
         }
         break;
       }
@@ -709,22 +740,8 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         outdata += sizeof (sync_bytes);
         memcpy (outdata, payload, nalu_size);
 
-        /* push codec_data first */
-        if (!rtph264depay->merge && rtph264depay->codec_data) {
-          gst_buffer_set_caps (rtph264depay->codec_data,
-              GST_PAD_CAPS (depayload->srcpad));
-          gst_base_rtp_depayload_push (depayload, rtph264depay->codec_data);
-          rtph264depay->codec_data = NULL;
-        }
-
-        if (rtph264depay->merge) {
-          outbuf = gst_rtp_h264_depay_push_nal (rtph264depay, outbuf, ts);
-          if (!outbuf)
-            break;
-        }
-
-        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-        return outbuf;
+        gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf, timestamp);
+        break;
       }
     }
   }
@@ -768,7 +785,7 @@ gst_rtp_h264_depay_change_state (GstElement * element,
       rtph264depay->wait_start = TRUE;
       gst_adapter_clear (rtph264depay->picture_adapter);
       rtph264depay->picture_start = FALSE;
-      rtph264depay->picture_complete = FALSE;
+      rtph264depay->last_keyframe = FALSE;
       break;
     default:
       break;
