@@ -29,14 +29,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtpmp4adepay_debug);
 #define GST_CAT_DEFAULT (rtpmp4adepay_debug)
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_mp4adepay_details =
-GST_ELEMENT_DETAILS ("RTP MPEG4 audio depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts MPEG4 audio from RTP packets (RFC 3016)",
-    "Nokia Corporation (contact <stefan.kost@nokia.com>), "
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_mp4a_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -87,7 +79,11 @@ gst_rtp_mp4a_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_mp4a_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_mp4adepay_details);
+  gst_element_class_set_details_simple (element_class,
+      "RTP MPEG4 audio depayloader", "Codec/Depayloader/Network",
+      "Extracts MPEG4 audio from RTP packets (RFC 3016)",
+      "Nokia Corporation (contact <stefan.kost@nokia.com>), "
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -169,6 +165,10 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
       guint8 *data;
       guint size;
       gint i;
+      guint sr_idx;
+      static const guint aac_sample_rates[] = { 96000, 88200, 64000, 48000,
+        44100, 32000, 24000, 22050, 16000, 12000, 11025, 8000
+      };
 
       buffer = gst_value_get_buffer (&v);
       gst_buffer_ref (buffer);
@@ -211,6 +211,17 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
         data[i] = ((data[i + 1] & 1) << 7) | ((data[i + 2] & 0xfe) >> 1);
       }
 
+      /* grab and set sampling rate */
+      sr_idx = ((data[0] & 0x07) << 1) | ((data[1] & 0x80) >> 7);
+      if (sr_idx < G_N_ELEMENTS (aac_sample_rates)) {
+        gst_caps_set_simple (srccaps,
+            "rate", G_TYPE_INT, (gint) aac_sample_rates[sr_idx], NULL);
+        GST_DEBUG_OBJECT (depayload, "sampling rate from stream-config %u",
+            aac_sample_rates[sr_idx]);
+      } else {
+        GST_WARNING_OBJECT (depayload, "Invalid sample rate index %u", sr_idx);
+      }
+
       /* ignore remaining bit, we're only interested in full bytes */
       GST_BUFFER_SIZE (buffer) = size;
 
