@@ -35,13 +35,6 @@ GST_DEBUG_CATEGORY_STATIC (rtpvorbispay_debug);
  * http://www.rfc-editor.org/rfc/rfc5215.txt
  */
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_vorbispay_details =
-GST_ELEMENT_DETAILS ("RTP Vorbis depayloader",
-    "Codec/Payloader/Network",
-    "Payload-encode Vorbis audio into RTP packets (RFC 5215)",
-    "Wim Taymans <wimi.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_vorbis_pay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -85,7 +78,10 @@ gst_rtp_vorbis_pay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_vorbis_pay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_vorbispay_details);
+  gst_element_class_set_details_simple (element_class, "RTP Vorbis depayloader",
+      "Codec/Payloader/Network",
+      "Payload-encode Vorbis audio into RTP packets (RFC 5215)",
+      "Wim Taymans <wimi.taymans@gmail.com>");
 }
 
 static void
@@ -97,8 +93,6 @@ gst_rtp_vorbis_pay_class_init (GstRtpVorbisPayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gstelement_class->change_state = gst_rtp_vorbis_pay_change_state;
 
   gstbasertppayload_class->set_caps = gst_rtp_vorbis_pay_setcaps;
