@@ -296,7 +296,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
 
   g_type_class_add_private (klass, sizeof (GstRtpJitterBufferPrivate));
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_finalize);
+  gobject_class->finalize = gst_rtp_jitter_buffer_finalize;
 
   gobject_class->set_property = gst_rtp_jitter_buffer_set_property;
   gobject_class->get_property = gst_rtp_jitter_buffer_get_property;
@@ -310,7 +310,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
   g_object_class_install_property (gobject_class, PROP_LATENCY,
       g_param_spec_uint ("latency", "Buffer latency in ms",
           "Amount of ms to buffer", 0, G_MAXUINT, DEFAULT_LATENCY_MS,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstRtpJitterBuffer::drop-on-latency:
    *
@@ -320,7 +320,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
       g_param_spec_boolean ("drop-on-latency",
           "Drop buffers when maximum latency is reached",
           "Tells the jitterbuffer to never exceed the given latency in size",
-          DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE));
+          DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstRtpJitterBuffer::ts-offset:
    *
@@ -469,6 +469,7 @@ gst_rtp_jitter_buffer_init (GstRtpJitterBuffer * jitterbuffer,
   rtp_jitter_buffer_reset_skew (priv->jbuf);
   rtp_jitter_buffer_set_delay (priv->jbuf, priv->latency_ns);
   rtp_jitter_buffer_set_buffering (priv->jbuf, FALSE);
+  priv->active = TRUE;
 
   priv->srcpad =
       gst_pad_new_from_static_template (&gst_rtp_jitter_buffer_src_template,
@@ -949,11 +950,6 @@ gst_rtp_jitter_buffer_change_state (GstElement * element,
       priv->last_pt = -1;
       /* block until we go to PLAYING */
       priv->blocked = TRUE;
-      /* reset skew detection initialy */
-      rtp_jitter_buffer_reset_skew (priv->jbuf);
-      rtp_jitter_buffer_set_delay (priv->jbuf, priv->latency_ns);
-      rtp_jitter_buffer_set_buffering (priv->jbuf, FALSE);
-      priv->active = TRUE;
       JBUF_UNLOCK (priv);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
@@ -1094,7 +1090,7 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
        * we are flushing */
       ret = priv->srcresult == GST_FLOW_OK;
       if (ret && !priv->eos) {
-        GST_DEBUG_OBJECT (jitterbuffer, "queuing EOS");
+        GST_INFO_OBJECT (jitterbuffer, "queuing EOS");
         priv->eos = TRUE;
         JBUF_SIGNAL (priv);
       } else if (priv->eos) {
@@ -1489,7 +1485,7 @@ eos_reached (GstClock * clock, GstClockTime time, GstClockID id,
 
   JBUF_LOCK_CHECK (priv, flushing);
   if (priv->waiting) {
-    GST_DEBUG_OBJECT (jitterbuffer, "got the NPT timeout");
+    GST_INFO_OBJECT (jitterbuffer, "got the NPT timeout");
     priv->reached_npt_stop = TRUE;
     JBUF_SIGNAL (priv);
   }
@@ -1505,6 +1501,34 @@ flushing:
   }
 }
 
+static GstClockTime
+compute_elapsed (GstRtpJitterBuffer * jitterbuffer, GstBuffer * outbuf)
+{
+  guint64 ext_time, elapsed;
+  guint32 rtp_time;
+  GstRtpJitterBufferPrivate *priv;
+
+  priv = jitterbuffer->priv;
+  rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
+
+  GST_LOG_OBJECT (jitterbuffer, "rtp %" G_GUINT32_FORMAT ", ext %"
+      G_GUINT64_FORMAT, rtp_time, priv->ext_timestamp);
+
+  if (rtp_time < priv->ext_timestamp) {
+    ext_time = priv->ext_timestamp;
+  } else {
+    ext_time = gst_rtp_buffer_ext_timestamp (&priv->ext_timestamp, rtp_time);
+  }
+
+  if (ext_time > priv->clock_base)
+    elapsed = ext_time - priv->clock_base;
+  else
+    elapsed = 0;
+
+  elapsed = gst_util_uint64_scale_int (elapsed, GST_SECOND, priv->clock_rate);
+  return elapsed;
+}
+
 /*
  * This funcion will push out buffers on the source pad.
  *
@@ -1539,8 +1563,37 @@ again:
     if (G_LIKELY (!priv->blocked)) {
       /* we're buffering but not EOS, wait. */
       if (!priv->eos && (!priv->active
-              || rtp_jitter_buffer_is_buffering (priv->jbuf)))
-        goto do_wait;
+              || rtp_jitter_buffer_is_buffering (priv->jbuf))) {
+        GstClockTime elapsed, delay, left;
+
+        if (priv->estimated_eos == -1)
+          goto do_wait;
+
+        outbuf = rtp_jitter_buffer_peek (priv->jbuf);
+        if (outbuf != NULL) {
+          elapsed = compute_elapsed (jitterbuffer, outbuf);
+          if (GST_BUFFER_DURATION_IS_VALID (outbuf))
+            elapsed += GST_BUFFER_DURATION (outbuf);
+        } else {
+          GST_INFO_OBJECT (jitterbuffer, "no buffer, using last_elapsed");
+          elapsed = priv->last_elapsed;
+        }
+
+        delay = rtp_jitter_buffer_get_delay (priv->jbuf);
+
+        if (priv->estimated_eos > elapsed)
+          left = priv->estimated_eos - elapsed;
+        else
+          left = 0;
+
+        GST_INFO_OBJECT (jitterbuffer, "buffering, elapsed %" GST_TIME_FORMAT
+            " estimated_eos %" GST_TIME_FORMAT " left %" GST_TIME_FORMAT
+            " delay %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (elapsed), GST_TIME_ARGS (priv->estimated_eos),
+            GST_TIME_ARGS (left), GST_TIME_ARGS (delay));
+        if (left > delay)
+          goto do_wait;
+      }
       /* if we have a packet, we can exit the loop and grab it */
       if (rtp_jitter_buffer_num_packets (priv->jbuf) > 0)
         break;
@@ -1555,7 +1608,7 @@ again:
         GST_OBJECT_LOCK (jitterbuffer);
         clock = GST_ELEMENT_CLOCK (jitterbuffer);
         if (clock) {
-          GST_DEBUG_OBJECT (jitterbuffer, "scheduling timeout");
+          GST_INFO_OBJECT (jitterbuffer, "scheduling timeout");
           id = gst_clock_new_single_shot_id (clock, sync_time);
           gst_clock_id_wait_async (id, (GstClockCallback) eos_reached,
               jitterbuffer);
@@ -1723,7 +1776,7 @@ again:
       GstEvent *event;
 
       /* we had a gap and thus we lost a packet. Create an event for this.  */
-      GST_WARNING_OBJECT (jitterbuffer, "Packet #%d lost", next_seqnum);
+      GST_DEBUG_OBJECT (jitterbuffer, "Packet #%d lost", next_seqnum);
       priv->num_late++;
       discont = TRUE;
 
@@ -1772,26 +1825,9 @@ push_buffer:
   /* update the elapsed time when we need to check against the npt stop time. */
   if (priv->npt_stop != -1 && priv->ext_timestamp != -1
       && priv->clock_base != -1 && priv->clock_rate > 0) {
-    guint64 ext_time, elapsed, estimated;
-    guint32 rtp_time;
+    guint64 elapsed, estimated;
 
-    rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
-
-    GST_LOG_OBJECT (jitterbuffer, "rtp %" G_GUINT32_FORMAT ", ext %"
-        G_GUINT64_FORMAT, rtp_time, priv->ext_timestamp);
-
-    if (rtp_time < priv->ext_timestamp) {
-      ext_time = priv->ext_timestamp;
-    } else {
-      ext_time = gst_rtp_buffer_ext_timestamp (&priv->ext_timestamp, rtp_time);
-    }
-
-    if (ext_time > priv->clock_base)
-      elapsed = ext_time - priv->clock_base;
-    else
-      elapsed = 0;
-
-    elapsed = gst_util_uint64_scale_int (elapsed, GST_SECOND, priv->clock_rate);
+    elapsed = compute_elapsed (jitterbuffer, outbuf);
 
     if (elapsed > priv->last_elapsed) {
       guint64 left;
@@ -1882,7 +1918,7 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   GstRtpJitterBuffer *jitterbuffer;
   GstRtpJitterBufferPrivate *priv;
   GstFlowReturn ret = GST_FLOW_OK;
-  guint64 base_rtptime, timestamp;
+  guint64 base_rtptime, base_time;
   guint32 clock_rate;
   guint64 last_rtptime;
   guint32 ssrc;
@@ -1920,14 +1956,14 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   ext_rtptime = gst_rtp_buffer_ext_timestamp (&ext_rtptime, rtptime);
 
   /* get the last values from the jitterbuffer */
-  rtp_jitter_buffer_get_sync (priv->jbuf, &base_rtptime, &timestamp,
+  rtp_jitter_buffer_get_sync (priv->jbuf, &base_rtptime, &base_time,
       &clock_rate, &last_rtptime);
 
   GST_DEBUG_OBJECT (jitterbuffer, "ext SR %" G_GUINT64_FORMAT ", base %"
       G_GUINT64_FORMAT ", clock-rate %" G_GUINT32_FORMAT,
       ext_rtptime, base_rtptime, clock_rate);
 
-  if (base_rtptime == -1 || clock_rate == -1 || timestamp == -1) {
+  if (base_rtptime == -1 || clock_rate == -1 || base_time == -1) {
     GST_DEBUG_OBJECT (jitterbuffer, "dropping, no RTP values");
     drop = TRUE;
   } else {
@@ -1958,7 +1994,7 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
 
     s = gst_structure_new ("application/x-rtp-sync",
         "base-rtptime", G_TYPE_UINT64, base_rtptime,
-        "base-time", G_TYPE_UINT64, timestamp,
+        "base-time", G_TYPE_UINT64, base_time,
         "clock-rate", G_TYPE_UINT, clock_rate,
         "sr-ext-rtptime", G_TYPE_UINT64, ext_rtptime,
         "sr-buffer", GST_TYPE_BUFFER, buffer, NULL);
