@@ -100,10 +100,25 @@ typedef struct {
   RTPSourceClockRate   clock_rate;
 } RTPSourceCallbacks;
 
+/**
+ * RTPConflictingAddress:
+ * @address: #GstNetAddress which conflicted
+ * @last_conflict_time: time when the last conflict was seen
+ *
+ * This structure is used to account for addresses that have conflicted to find
+ * loops.
+ */
+typedef struct {
+  GstNetAddress address;
+  GstClockTime time;
+} RTPConflictingAddress;
+
 /**
  * RTPSource:
  *
  * A source in the #RTPSession
+ *
+ * @conflicting_addresses: GList of conflicting addresses
  */
 struct _RTPSource {
   GObject       object;
@@ -116,6 +131,7 @@ struct _RTPSource {
   gboolean      internal;
   gboolean      is_csrc;
   gboolean      is_sender;
+  gboolean      closing;
 
   GstStructure  *sdes;
 
@@ -136,13 +152,14 @@ struct _RTPSource {
   GstClockTime  last_activity;
   GstClockTime  last_rtp_activity;
 
+  GstClockTime  last_rtime;
   GstClockTime  last_rtptime;
-  GstClockTime  last_ntpnstime;
 
   /* for bitrate estimation */
   guint64       bitrate;
-  GstClockTime  prev_ntpnstime;
+  GstClockTime  prev_rtime;
   guint64       bytes_sent;
+  guint64       bytes_received;
 
   GQueue       *packets;
 
@@ -150,6 +167,8 @@ struct _RTPSource {
   gpointer           user_data;
 
   RTPSourceStats stats;
+
+  GList         *conflicting_addresses;
 };
 
 struct _RTPSourceClass {
@@ -192,8 +211,8 @@ void            rtp_source_set_rtcp_from       (RTPSource *src, GstNetAddress *a
 /* handling RTP */
 GstFlowReturn   rtp_source_process_rtp         (RTPSource *src, GstBuffer *buffer, RTPArrivalStats *arrival);
 
-GstFlowReturn   rtp_source_send_rtp            (RTPSource *src, gpointer data, gboolean is_list, guint64 ntpnstime);
-
+GstFlowReturn   rtp_source_send_rtp            (RTPSource *src, gpointer data, gboolean is_list,
+                                                GstClockTime running_time);
 /* RTCP messages */
 void            rtp_source_process_bye         (RTPSource *src, const gchar *reason);
 void            rtp_source_process_sr          (RTPSource *src, GstClockTime time, guint64 ntptime,
@@ -202,8 +221,8 @@ void            rtp_source_process_rb          (RTPSource *src, GstClockTime tim
                                                 gint32 packetslost, guint32 exthighestseq, guint32 jitter,
                                                 guint32 lsr, guint32 dlsr);
 
-gboolean        rtp_source_get_new_sr          (RTPSource *src, guint64 ntpnstime, guint64 *ntptime,
-		                                guint32 *rtptime, guint32 *packet_count,
+gboolean        rtp_source_get_new_sr          (RTPSource *src, guint64 ntpnstime, GstClockTime running_time,
+                                                guint64 *ntptime, guint32 *rtptime, guint32 *packet_count,
 						guint32 *octet_count);
 gboolean        rtp_source_get_new_rb          (RTPSource *src, GstClockTime time, guint8 *fractionlost,
                                                 gint32 *packetslost, guint32 *exthighestseq, guint32 *jitter,
@@ -218,4 +237,17 @@ gboolean        rtp_source_get_last_rb         (RTPSource *src, guint8 *fraction
 
 void            rtp_source_reset               (RTPSource * src);
 
+gboolean        rtp_source_find_conflicting_address (RTPSource * src,
+                                                GstNetAddress *address,
+                                                GstClockTime time);
+
+void            rtp_source_add_conflicting_address (RTPSource * src,
+                                                GstNetAddress *address,
+                                                GstClockTime time);
+
+void            rtp_source_timeout             (RTPSource * src,
+                                                GstClockTime current_time,
+                                                GstClockTime collision_timeout);
+
+
 #endif /* __RTP_SOURCE_H__ */
