@@ -188,6 +188,7 @@ struct _GstRTSPSrc {
   GStaticRecMutex *conn_rec_lock;
 
   GstSDPMessage   *sdp;
+  gboolean         from_sdp;
   gint             numstreams;
   GList           *streams;
   GstStructure    *props;
@@ -213,6 +214,7 @@ struct _GstRTSPSrc {
   gchar            *user_pw;
   gint              buffer_mode;
   GstRTSPRange      client_port_range;
+  gint              udp_buffer_size;
 
   /* state */
   GstRTSPState       state;
