@@ -55,6 +55,7 @@
 #include "gstwavparse.h"
 #include "gst/riff/riff-ids.h"
 #include "gst/riff/riff-media.h"
+#include <gst/base/gsttypefindhelper.h>
 #include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_STATIC (wavparse_debug);
@@ -81,12 +82,6 @@ static gboolean gst_wavparse_sink_event (GstPad * pad, GstEvent * event);
 static void gst_wavparse_loop (GstPad * pad);
 static gboolean gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event);
 
-static const GstElementDetails gst_wavparse_details =
-GST_ELEMENT_DETAILS ("WAV audio demuxer",
-    "Codec/Demuxer/Audio",
-    "Parse a .wav file into raw audio",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 static GstStaticPadTemplate sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("wavparse_sink",
     GST_PAD_SINK,
@@ -115,7 +110,10 @@ gst_wavparse_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class, src_template);
   gst_object_unref (src_template);
 
-  gst_element_class_set_details (element_class, &gst_wavparse_details);
+  gst_element_class_set_details_simple (element_class, "WAV audio demuxer",
+      "Codec/Demuxer/Audio",
+      "Parse a .wav file into raw audio",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
 }
 
 static void
@@ -168,6 +166,15 @@ gst_wavparse_reset (GstWavParse * wav)
   if (wav->tags)
     gst_tag_list_free (wav->tags);
   wav->tags = NULL;
+  if (wav->caps)
+    gst_caps_unref (wav->caps);
+  wav->caps = NULL;
+  if (wav->start_segment)
+    gst_event_unref (wav->start_segment);
+  wav->start_segment = NULL;
+  if (wav->close_segment)
+    gst_event_unref (wav->close_segment);
+  wav->close_segment = NULL;
 }
 
 static void
@@ -655,15 +662,16 @@ gst_wavparse_other (GstWavParse * wav)
           length = G_MAXUINT32;
         }
         if (file_length > G_MAXUINT32) {
-          GST_DEBUG_OBJECT (wav, "file length %lld, clipping to 32 bits");
+          GST_DEBUG_OBJECT (wav, "file length %" G_GUINT64_FORMAT
+              ", clipping to 32 bits", file_length);
           /* could not get length, assuming till eof */
           length = G_MAXUINT32;
         } else {
-          GST_DEBUG_OBJECT (wav, "file length %lld, datalength",
-              file_length, length);
+          GST_DEBUG_OBJECT (wav, "file length %" G_GUINT64_FORMAT
+              ", datalength %u", file_length, length);
           /* substract offset of datastart from length */
           length = file_length - wav->datastart;
-          GST_DEBUG_OBJECT (wav, "datalength %lld", length);
+          GST_DEBUG_OBJECT (wav, "datalength %u", length);
         }
       }
       wav->datasize = (guint64) length;
@@ -956,17 +964,14 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
       /* we are running the current segment and doing a non-flushing seek,
        * close the segment first based on the previous last_stop. */
       GST_DEBUG_OBJECT (wav, "closing running segment %" G_GINT64_FORMAT
-          " to %" G_GINT64_FORMAT, wav->segment.accum, wav->segment.last_stop);
+          " to %" G_GINT64_FORMAT, wav->segment.start, wav->segment.last_stop);
 
       /* queue the segment for sending in the stream thread */
       if (wav->close_segment)
         gst_event_unref (wav->close_segment);
       wav->close_segment = gst_event_new_new_segment (TRUE,
           wav->segment.rate, wav->segment.format,
-          wav->segment.accum, wav->segment.last_stop, wav->segment.accum);
-
-      /* keep track of our last_stop */
-      seeksegment.accum = wav->segment.last_stop;
+          wav->segment.start, wav->segment.last_stop, wav->segment.start);
     }
   }
 
@@ -1075,10 +1080,20 @@ gst_wavparse_peek_chunk (GstWavParse * wav, guint32 * tag, guint32 * size)
   if (!gst_wavparse_peek_chunk_info (wav, tag, size))
     return FALSE;
 
-  GST_DEBUG ("Need to peek chunk of %d bytes", *size);
+  /* size 0 -> empty data buffer would surprise most callers,
+   * large size -> do not bother trying to squeeze that into adapter,
+   * so we throw poor man's exception, which can be caught if caller really
+   * wants to handle 0 size chunk */
+  if (!(*size) || (*size) >= (1 << 30)) {
+    GST_INFO ("Invalid/unexpected chunk size %d for tag %" GST_FOURCC_FORMAT,
+        *size, GST_FOURCC_ARGS (*tag));
+    /* chain should give up */
+    wav->abort_buffering = TRUE;
+    return FALSE;
+  }
   peek_size = (*size + 1) & ~1;
-
   available = gst_adapter_available (wav->adapter);
+
   if (available >= (8 + peek_size)) {
     return TRUE;
   } else {
@@ -1118,7 +1133,7 @@ gst_wavparse_calculate_duration (GstWavParse * wav)
   return FALSE;
 }
 
-static void
+static gboolean
 gst_waveparse_ignore_chunk (GstWavParse * wav, GstBuffer * buf, guint32 tag,
     guint32 size)
 {
@@ -1126,7 +1141,7 @@ gst_waveparse_ignore_chunk (GstWavParse * wav, GstBuffer * buf, guint32 tag,
 
   if (wav->streaming) {
     if (!gst_wavparse_peek_chunk (wav, &tag, &size))
-      return;
+      return FALSE;
   }
   GST_DEBUG_OBJECT (wav, "Ignoring tag %" GST_FOURCC_FORMAT,
       GST_FOURCC_ARGS (tag));
@@ -1137,17 +1152,19 @@ gst_waveparse_ignore_chunk (GstWavParse * wav, GstBuffer * buf, guint32 tag,
   } else {
     gst_buffer_unref (buf);
   }
+
+  return TRUE;
 }
 
 static GstFlowReturn
 gst_wavparse_stream_headers (GstWavParse * wav)
 {
-  GstFlowReturn res;
-  GstBuffer *buf;
+  GstFlowReturn res = GST_FLOW_OK;
+  GstBuffer *buf = NULL;
   gst_riff_strf_auds *header = NULL;
   guint32 tag, size;
   gboolean gotdata = FALSE;
-  GstCaps *caps;
+  GstCaps *caps = NULL;
   gchar *codec_name = NULL;
   GstEvent **event_p;
   GstFormat bformat;
@@ -1160,20 +1177,28 @@ gst_wavparse_stream_headers (GstWavParse * wav)
     /* The header starts with a 'fmt ' tag */
     if (wav->streaming) {
       if (!gst_wavparse_peek_chunk (wav, &tag, &size))
-        return GST_FLOW_OK;
+        return res;
 
       gst_adapter_flush (wav->adapter, 8);
       wav->offset += 8;
 
-      buf = gst_adapter_take_buffer (wav->adapter, size);
+      if (size) {
+        buf = gst_adapter_take_buffer (wav->adapter, size);
+        if (size & 1)
+          gst_adapter_flush (wav->adapter, 1);
+        wav->offset += GST_ROUND_UP_2 (size);
+      } else {
+        buf = gst_buffer_new ();
+      }
     } else {
       if ((res = gst_riff_read_chunk (GST_ELEMENT_CAST (wav), wav->sinkpad,
                   &wav->offset, &tag, &buf)) != GST_FLOW_OK)
         return res;
     }
 
-    if (tag == GST_RIFF_TAG_JUNK || tag == GST_RIFF_TAG_bext ||
-        tag == GST_RIFF_TAG_BEXT || tag == GST_RIFF_TAG_LIST) {
+    if (tag == GST_RIFF_TAG_JUNK || tag == GST_RIFF_TAG_JUNQ ||
+        tag == GST_RIFF_TAG_bext || tag == GST_RIFF_TAG_BEXT ||
+        tag == GST_RIFF_TAG_LIST) {
       GST_DEBUG_OBJECT (wav, "skipping %" GST_FOURCC_FORMAT " chunk",
           GST_FOURCC_ARGS (tag));
       gst_buffer_unref (buf);
@@ -1295,7 +1320,7 @@ gst_wavparse_stream_headers (GstWavParse * wav)
   while (!gotdata) {
     if (wav->streaming) {
       if (!gst_wavparse_peek_chunk_info (wav, &tag, &size))
-        return GST_FLOW_OK;
+        goto exit;
     } else {
       if ((res =
               gst_pad_pull_range (wav->sinkpad, wav->offset, 8,
@@ -1323,8 +1348,13 @@ gst_wavparse_stream_headers (GstWavParse * wav)
         }
         wav->offset += 8;
         wav->datastart = wav->offset;
-        /* file might be truncated */
-        if (upstream_size) {
+        /* If size is zero, then the data chunk probably actually extends to
+           the end of the file */
+        if (size == 0 && upstream_size) {
+          size = upstream_size - wav->datastart;
+        }
+        /* Or the file might be truncated */
+        else if (upstream_size) {
           size = MIN (size, (upstream_size - wav->datastart));
         }
         wav->datasize = (guint64) size;
@@ -1342,17 +1372,27 @@ gst_wavparse_stream_headers (GstWavParse * wav)
             wav->format != GST_RIFF_WAVE_FORMAT_MPEGL3) {
           const guint data_size = 4;
 
+          GST_INFO_OBJECT (wav, "Have fact chunk");
+          if (size < data_size) {
+            if (!gst_waveparse_ignore_chunk (wav, buf, tag, size)) {
+              /* need more data */
+              goto exit;
+            }
+            GST_DEBUG_OBJECT (wav, "need %d, available %d; ignoring chunk",
+                data_size, size);
+            break;
+          }
           /* number of samples (for compressed formats) */
           if (wav->streaming) {
             const guint8 *data = NULL;
 
-            if (gst_adapter_available (wav->adapter) < 8 + data_size) {
-              return GST_FLOW_OK;
+            if (!gst_wavparse_peek_chunk (wav, &tag, &size)) {
+              goto exit;
             }
             gst_adapter_flush (wav->adapter, 8);
             data = gst_adapter_peek (wav->adapter, data_size);
             wav->fact = GST_READ_UINT32_LE (data);
-            gst_adapter_flush (wav->adapter, data_size);
+            gst_adapter_flush (wav->adapter, GST_ROUND_UP_2 (size));
           } else {
             gst_buffer_unref (buf);
             if ((res =
@@ -1363,10 +1403,13 @@ gst_wavparse_stream_headers (GstWavParse * wav)
             gst_buffer_unref (buf);
           }
           GST_DEBUG_OBJECT (wav, "have fact %u", wav->fact);
-          wav->offset += 8 + data_size;
+          wav->offset += 8 + GST_ROUND_UP_2 (size);
           break;
         } else {
-          gst_waveparse_ignore_chunk (wav, buf, tag, size);
+          if (!gst_waveparse_ignore_chunk (wav, buf, tag, size)) {
+            /* need more data */
+            goto exit;
+          }
         }
         break;
       }
@@ -1374,9 +1417,19 @@ gst_wavparse_stream_headers (GstWavParse * wav)
         const gst_riff_acid *acid = NULL;
         const guint data_size = sizeof (gst_riff_acid);
 
+        GST_INFO_OBJECT (wav, "Have acid chunk");
+        if (size < data_size) {
+          if (!gst_waveparse_ignore_chunk (wav, buf, tag, size)) {
+            /* need more data */
+            goto exit;
+          }
+          GST_DEBUG_OBJECT (wav, "need %d, available %d; ignoring chunk",
+              data_size, size);
+          break;
+        }
         if (wav->streaming) {
-          if (gst_adapter_available (wav->adapter) < 8 + data_size) {
-            return GST_FLOW_OK;
+          if (!gst_wavparse_peek_chunk (wav, &tag, &size)) {
+            goto exit;
           }
           gst_adapter_flush (wav->adapter, 8);
           acid = (const gst_riff_acid *) gst_adapter_peek (wav->adapter,
@@ -1385,23 +1438,23 @@ gst_wavparse_stream_headers (GstWavParse * wav)
           gst_buffer_unref (buf);
           if ((res =
                   gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
-                      data_size, &buf)) != GST_FLOW_OK)
+                      size, &buf)) != GST_FLOW_OK)
             goto header_read_error;
           acid = (const gst_riff_acid *) GST_BUFFER_DATA (buf);
         }
-        GST_INFO_OBJECT (wav, "Have acid chunk");
         /* send data as tags */
         if (!wav->tags)
           wav->tags = gst_tag_list_new ();
         gst_tag_list_add (wav->tags, GST_TAG_MERGE_REPLACE,
             GST_TAG_BEATS_PER_MINUTE, acid->tempo, NULL);
 
+        size = GST_ROUND_UP_2 (size);
         if (wav->streaming) {
-          gst_adapter_flush (wav->adapter, data_size);
+          gst_adapter_flush (wav->adapter, size);
         } else {
           gst_buffer_unref (buf);
-          wav->offset += 8 + data_size;
         }
+        wav->offset += 8 + size;
         break;
       }
         /* FIXME: all list tags after data are ignored in streaming mode */
@@ -1412,7 +1465,7 @@ gst_wavparse_stream_headers (GstWavParse * wav)
           const guint8 *data = NULL;
 
           if (gst_adapter_available (wav->adapter) < 12) {
-            return GST_FLOW_OK;
+            goto exit;
           }
           data = gst_adapter_peek (wav->adapter, 12);
           ltag = GST_READ_UINT32_LE (data + 8);
@@ -1426,18 +1479,21 @@ gst_wavparse_stream_headers (GstWavParse * wav)
         }
         switch (ltag) {
           case GST_RIFF_LIST_INFO:{
-            const guint data_size = size - 4;
+            const gint data_size = size - 4;
             GstTagList *new;
 
             GST_INFO_OBJECT (wav, "Have LIST chunk INFO size %u", data_size);
             if (wav->streaming) {
-              gst_adapter_flush (wav->adapter, 12);
-              if (gst_adapter_available (wav->adapter) < data_size) {
-                return GST_FLOW_OK;
+              if (!gst_wavparse_peek_chunk (wav, &tag, &size)) {
+                goto exit;
               }
-              gst_buffer_unref (buf);
-              if (data_size > 0)
+              gst_adapter_flush (wav->adapter, 12);
+              wav->offset += 12;
+              if (data_size > 0) {
                 buf = gst_adapter_take_buffer (wav->adapter, data_size);
+                if (data_size & 1)
+                  gst_adapter_flush (wav->adapter, 1);
+              }
             } else {
               wav->offset += 12;
               gst_buffer_unref (buf);
@@ -1459,25 +1515,26 @@ gst_wavparse_stream_headers (GstWavParse * wav)
                   gst_tag_list_free (old);
                 gst_tag_list_free (new);
               }
-              if (wav->streaming) {
-                gst_adapter_flush (wav->adapter, data_size);
-              } else {
-                gst_buffer_unref (buf);
-                wav->offset += data_size;
-              }
+              gst_buffer_unref (buf);
+              wav->offset += GST_ROUND_UP_2 (data_size);
             }
             break;
           }
           default:
             GST_INFO_OBJECT (wav, "Ignoring LIST chunk %" GST_FOURCC_FORMAT,
                 GST_FOURCC_ARGS (ltag));
-            gst_waveparse_ignore_chunk (wav, buf, tag, size);
+            if (!gst_waveparse_ignore_chunk (wav, buf, tag, size))
+              /* need more data */
+              goto exit;
             break;
         }
         break;
       }
       default:
-        gst_waveparse_ignore_chunk (wav, buf, tag, size);
+        if (!gst_waveparse_ignore_chunk (wav, buf, tag, size))
+          /* need more data */
+          goto exit;
+        break;
     }
 
     if (upstream_size && (wav->offset >= upstream_size)) {
@@ -1524,75 +1581,78 @@ gst_wavparse_stream_headers (GstWavParse * wav)
   return GST_FLOW_OK;
 
   /* ERROR */
+exit:
+  {
+    if (codec_name)
+      g_free (codec_name);
+    if (header)
+      g_free (header);
+    if (caps)
+      gst_caps_unref (caps);
+    return res;
+  }
+fail:
+  {
+    res = GST_FLOW_ERROR;
+    goto exit;
+  }
 invalid_wav:
   {
     GST_ELEMENT_ERROR (wav, STREAM, TYPE_NOT_FOUND, (NULL),
         ("Invalid WAV header (no fmt at start): %"
             GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag)));
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 parse_header_error:
   {
     GST_ELEMENT_ERROR (wav, STREAM, DEMUX, (NULL),
         ("Couldn't parse audio header"));
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 no_channels:
   {
     GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
         ("Stream claims to contain no channels - invalid data"));
-    g_free (header);
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 no_rate:
   {
     GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
         ("Stream with sample_rate == 0 - invalid data"));
-    g_free (header);
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 invalid_blockalign:
   {
     GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
         ("Stream claims blockalign = %u, which is more than %u - invalid data",
             wav->blockalign, wav->channels * (guint) ceil (wav->depth / 8.0)));
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 invalid_bps:
   {
     GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
         ("Stream claims av_bsp = %u, which is more than %u - invalid data",
             wav->av_bps, wav->blockalign * wav->rate));
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 no_bytes_per_sample:
   {
     GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
         ("Could not caluclate bytes per sample - invalid data"));
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 unknown_format:
   {
     GST_ELEMENT_ERROR (wav, STREAM, TYPE_NOT_FOUND, (NULL),
         ("No caps found for format 0x%x, %d channels, %d Hz",
             wav->format, wav->channels, wav->rate));
-    g_free (header);
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 header_read_error:
   {
     GST_ELEMENT_ERROR (wav, STREAM, DEMUX, (NULL),
         ("Couldn't read in header %d (%s)", res, gst_flow_get_name (res)));
-    g_free (codec_name);
-    return GST_FLOW_ERROR;
+    goto fail;
   }
 }
 
@@ -1667,22 +1727,32 @@ static void
 gst_wavparse_add_src_pad (GstWavParse * wav, GstBuffer * buf)
 {
   GstStructure *s;
-  const guint8 dts_marker[] = { 0xFF, 0x1F, 0x00, 0xE8, 0xF1, 0x07 };
 
   GST_DEBUG_OBJECT (wav, "adding src pad");
 
   if (wav->caps) {
     s = gst_caps_get_structure (wav->caps, 0);
-    if (s && gst_structure_has_name (s, "audio/x-raw-int") && buf &&
-        GST_BUFFER_SIZE (buf) > 6 &&
-        memcmp (GST_BUFFER_DATA (buf), dts_marker, 6) == 0) {
+    if (s && gst_structure_has_name (s, "audio/x-raw-int") && buf != NULL) {
+      GstTypeFindProbability prob;
+      GstCaps *tf_caps;
 
-      GST_WARNING_OBJECT (wav, "Found DTS marker in file marked as raw PCM");
-      gst_caps_unref (wav->caps);
-      wav->caps = gst_caps_from_string ("audio/x-dts");
+      tf_caps = gst_type_find_helper_for_buffer (GST_OBJECT (wav), buf, &prob);
+      if (tf_caps != NULL) {
+        s = gst_caps_get_structure (tf_caps, 0);
+        if (gst_structure_has_name (s, "audio/x-dts")
+            && prob >= GST_TYPE_FIND_LIKELY) {
+          GST_INFO_OBJECT (wav, "Found DTS marker in file marked as raw PCM");
+          gst_caps_unref (wav->caps);
+          wav->caps = tf_caps;
 
-      gst_tag_list_add (wav->tags, GST_TAG_MERGE_REPLACE,
-          GST_TAG_AUDIO_CODEC, "dts", NULL);
+          gst_tag_list_add (wav->tags, GST_TAG_MERGE_REPLACE,
+              GST_TAG_AUDIO_CODEC, "dts", NULL);
+        } else {
+          GST_DEBUG_OBJECT (wav, "found caps %" GST_PTR_FORMAT " for stream "
+              "marked as raw PCM audio, but ignoring for now", tf_caps);
+          gst_caps_unref (tf_caps);
+        }
+      }
     }
   }
 
@@ -1783,8 +1853,30 @@ iterate_adapter:
     if ((res = gst_pad_pull_range (wav->sinkpad, wav->offset,
                 desired, &buf)) != GST_FLOW_OK)
       goto pull_error;
+
+    /* we may get a short buffer at the end of the file */
+    if (GST_BUFFER_SIZE (buf) < desired) {
+      GST_LOG_OBJECT (wav, "Got only %u bytes of data", GST_BUFFER_SIZE (buf));
+      if (GST_BUFFER_SIZE (buf) >= wav->blockalign) {
+        buf = gst_buffer_make_metadata_writable (buf);
+        GST_BUFFER_SIZE (buf) -= (GST_BUFFER_SIZE (buf) % wav->blockalign);
+      } else {
+        gst_buffer_unref (buf);
+        goto found_eos;
+      }
+    }
   }
 
+  obtained = GST_BUFFER_SIZE (buf);
+
+  /* our positions in bytes */
+  pos = wav->offset - wav->datastart;
+  nextpos = pos + obtained;
+
+  /* update offsets, does not overflow. */
+  GST_BUFFER_OFFSET (buf) = pos / wav->bytes_per_sample;
+  GST_BUFFER_OFFSET_END (buf) = nextpos / wav->bytes_per_sample;
+
   /* first chunk of data? create the source pad. We do this only here so
    * we can detect broken .wav files with dts disguised as raw PCM (sigh) */
   if (G_UNLIKELY (wav->first)) {
@@ -1803,16 +1895,6 @@ iterate_adapter:
     }
   }
 
-  obtained = GST_BUFFER_SIZE (buf);
-
-  /* our positions in bytes */
-  pos = wav->offset - wav->datastart;
-  nextpos = pos + obtained;
-
-  /* update offsets, does not overflow. */
-  GST_BUFFER_OFFSET (buf) = pos / wav->bytes_per_sample;
-  GST_BUFFER_OFFSET_END (buf) = nextpos / wav->bytes_per_sample;
-
   if (wav->bps > 0) {
     /* and timestamps if we have a bitrate, be careful for overflows */
     timestamp = uint64_ceiling_scale (pos, GST_SECOND, (guint64) wav->bps);
@@ -1821,7 +1903,9 @@ iterate_adapter:
     duration = next_timestamp - timestamp;
 
     /* update current running segment position */
-    gst_segment_set_last_stop (&wav->segment, GST_FORMAT_TIME, next_timestamp);
+    if (G_LIKELY (next_timestamp >= wav->segment.start))
+      gst_segment_set_last_stop (&wav->segment, GST_FORMAT_TIME,
+          next_timestamp);
   } else if (wav->fact) {
     guint64 bps =
         gst_util_uint64_scale_int (wav->datasize, wav->rate, wav->fact);
@@ -1838,7 +1922,8 @@ iterate_adapter:
       timestamp = GST_CLOCK_TIME_NONE;
     duration = GST_CLOCK_TIME_NONE;
     /* update current running segment position with byte offset */
-    gst_segment_set_last_stop (&wav->segment, GST_FORMAT_BYTES, nextpos);
+    if (G_LIKELY (nextpos >= wav->segment.start))
+      gst_segment_set_last_stop (&wav->segment, GST_FORMAT_BYTES, nextpos);
   }
   if ((pos > 0) && wav->vbr) {
     /* don't set timestamps for VBR files if it's not the first buffer */
@@ -1953,36 +2038,39 @@ pause:
     wav->segment_running = FALSE;
     gst_pad_pause_task (pad);
 
-    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-      if (ret == GST_FLOW_UNEXPECTED) {
-        /* add pad before we perform EOS */
-        if (G_UNLIKELY (wav->first)) {
-          wav->first = FALSE;
-          gst_wavparse_add_src_pad (wav, NULL);
-        }
-        /* perform EOS logic */
-        if (wav->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-          GstClockTime stop;
+    if (ret == GST_FLOW_UNEXPECTED) {
+      /* add pad before we perform EOS */
+      if (G_UNLIKELY (wav->first)) {
+        wav->first = FALSE;
+        gst_wavparse_add_src_pad (wav, NULL);
+      }
 
-          if ((stop = wav->segment.stop) == -1)
-            stop = wav->segment.duration;
+      if (wav->state == GST_WAVPARSE_START)
+        GST_ELEMENT_ERROR (wav, STREAM, WRONG_TYPE,
+            ("No valid input found before end of stream"), (NULL));
 
-          gst_element_post_message (GST_ELEMENT_CAST (wav),
-              gst_message_new_segment_done (GST_OBJECT_CAST (wav),
-                  wav->segment.format, stop));
-        } else {
-          if (wav->srcpad != NULL)
-            gst_pad_push_event (wav->srcpad, gst_event_new_eos ());
-        }
+      /* perform EOS logic */
+      if (wav->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+        GstClockTime stop;
+
+        if ((stop = wav->segment.stop) == -1)
+          stop = wav->segment.duration;
+
+        gst_element_post_message (GST_ELEMENT_CAST (wav),
+            gst_message_new_segment_done (GST_OBJECT_CAST (wav),
+                wav->segment.format, stop));
       } else {
-        /* for fatal errors we post an error message, post the error
-         * first so the app knows about the error first. */
-        GST_ELEMENT_ERROR (wav, STREAM, FAILED,
-            (_("Internal data flow error.")),
-            ("streaming task paused, reason %s (%d)", reason, ret));
         if (wav->srcpad != NULL)
           gst_pad_push_event (wav->srcpad, gst_event_new_eos ());
       }
+    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
+      /* for fatal errors we post an error message, post the error
+       * first so the app knows about the error first. */
+      GST_ELEMENT_ERROR (wav, STREAM, FAILED,
+          (_("Internal data flow error.")),
+          ("streaming task paused, reason %s (%d)", reason, ret));
+      if (wav->srcpad != NULL)
+        gst_pad_push_event (wav->srcpad, gst_event_new_eos ());
     }
     return;
   }
@@ -2030,6 +2118,13 @@ gst_wavparse_chain (GstPad * pad, GstBuffer * buf)
       g_return_val_if_reached (GST_FLOW_ERROR);
   }
 done:
+  if (G_UNLIKELY (wav->abort_buffering)) {
+    wav->abort_buffering = FALSE;
+    ret = GST_FLOW_ERROR;
+    /* sort of demux/parse error */
+    GST_ELEMENT_ERROR (wav, STREAM, DEMUX, (NULL), ("unhandled buffer size"));
+  }
+
   return ret;
 }
 
@@ -2144,8 +2239,19 @@ gst_wavparse_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
-      /* stream leftover data in current segment */
-      gst_wavparse_flush_data (wav);
+      /* add pad if needed so EOS is seen downstream */
+      if (G_UNLIKELY (wav->first)) {
+        wav->first = FALSE;
+        gst_wavparse_add_src_pad (wav, NULL);
+      } else {
+        /* stream leftover data in current segment */
+        gst_wavparse_flush_data (wav);
+      }
+
+      if (wav->state == GST_WAVPARSE_START)
+        GST_ELEMENT_ERROR (wav, STREAM, WRONG_TYPE,
+            ("No valid input found before end of stream"), (NULL));
+
       /* fall-through */
     case GST_EVENT_FLUSH_STOP:
       gst_adapter_clear (wav->adapter);
@@ -2440,23 +2546,20 @@ gst_wavparse_sink_activate (GstPad * sinkpad)
   GstWavParse *wav = GST_WAVPARSE (gst_pad_get_parent (sinkpad));
   gboolean res;
 
-  if (wav->adapter)
-    gst_object_unref (wav->adapter);
+  if (wav->adapter) {
+    gst_adapter_clear (wav->adapter);
+    g_object_unref (wav->adapter);
+    wav->adapter = NULL;
+  }
 
   if (gst_pad_check_pull_range (sinkpad)) {
     GST_DEBUG ("going to pull mode");
     wav->streaming = FALSE;
-    if (wav->adapter) {
-      gst_adapter_clear (wav->adapter);
-      g_object_unref (wav->adapter);
-    }
-    wav->adapter = NULL;
     res = gst_pad_activate_pull (sinkpad, TRUE);
   } else {
     GST_DEBUG ("going to push (streaming) mode");
     wav->streaming = TRUE;
-    if (wav->adapter == NULL)
-      wav->adapter = gst_adapter_new ();
+    wav->adapter = gst_adapter_new ();
     res = gst_pad_activate_push (sinkpad, TRUE);
   }
   gst_object_unref (wav);
