@@ -53,13 +53,6 @@
 
 #include <string.h>
 
-
-static const GstElementDetails gst_v4l2sink_details =
-GST_ELEMENT_DETAILS ("Video (video4linux2) Sink",
-    "Sink/Video",
-    "Displays frames on a video4linux2 device",
-    "Rob Clark <rob@ti.com>,");
-
 GST_DEBUG_CATEGORY (v4l2sink_debug);
 #define GST_CAT_DEFAULT v4l2sink_debug
 
@@ -119,7 +112,7 @@ gst_v4l2sink_interface_init (GstImplementsInterfaceClass * klass)
   klass->supported = gst_v4l2sink_iface_supported;
 }
 
-void
+static void
 gst_v4l2sink_init_interfaces (GType type)
 {
   static const GInterfaceInfo v4l2iface_info = {
@@ -201,7 +194,9 @@ gst_v4l2sink_base_init (gpointer g_class)
 
   GST_DEBUG_CATEGORY_INIT (v4l2sink_debug, "v4l2sink", 0, "V4L2 sink element");
 
-  gst_element_class_set_details (gstelement_class, &gst_v4l2sink_details);
+  gst_element_class_set_details_simple (gstelement_class,
+      "Video (video4linux2) Sink", "Sink/Video",
+      "Displays frames on a video4linux2 device", "Rob Clark <rob@ti.com>,");
 
   gst_element_class_add_pad_template
       (gstelement_class,
@@ -233,23 +228,23 @@ gst_v4l2sink_class_init (GstV4l2SinkClass * klass)
       g_param_spec_uint ("queue-size", "Queue size",
           "Number of buffers to be enqueud in the driver in streaming mode",
           GST_V4L2_MIN_BUFFERS, GST_V4L2_MAX_BUFFERS, PROP_DEF_QUEUE_SIZE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_TOP,
       g_param_spec_int ("overlay-top", "Overlay top",
           "The topmost (y) coordinate of the video overlay; top left corner of screen is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_LEFT,
       g_param_spec_int ("overlay-left", "Overlay left",
           "The leftmost (x) coordinate of the video overlay; top left corner of screen is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_WIDTH,
       g_param_spec_uint ("overlay-width", "Overlay width",
           "The width of the video overlay; default is equal to negotiated image width",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_HEIGHT,
       g_param_spec_uint ("overlay-height", "Overlay height",
           "The height of the video overlay; default is equal to negotiated image height",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   basesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_v4l2sink_get_caps);
   basesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_v4l2sink_set_caps);
@@ -309,7 +304,7 @@ gst_v4l2sink_finalize (GstV4l2Sink * v4l2sink)
 }
 
 
-/**
+/*
  * State values
  */
 enum
@@ -319,7 +314,7 @@ enum
   STATE_STREAMING
 };
 
-/**
+/*
  * flags to indicate which overlay properties the user has set (and therefore
  * which ones should override the defaults from the driver)
  */
@@ -334,6 +329,9 @@ enum
 static void
 gst_v4l2sink_sync_overlay_fields (GstV4l2Sink * v4l2sink)
 {
+  if (!v4l2sink->overlay_fields_set)
+    return;
+
   if (GST_V4L2_IS_OPEN (v4l2sink->v4l2object)) {
 
     gint fd = v4l2sink->v4l2object->video_fd;
@@ -344,19 +342,17 @@ gst_v4l2sink_sync_overlay_fields (GstV4l2Sink * v4l2sink)
 
     g_return_if_fail (v4l2_ioctl (fd, VIDIOC_G_FMT, &format) >= 0);
 
-    if (v4l2sink->overlay_fields_set) {
-      if (v4l2sink->overlay_fields_set & OVERLAY_TOP_SET)
-        format.fmt.win.w.top = v4l2sink->overlay.top;
-      if (v4l2sink->overlay_fields_set & OVERLAY_LEFT_SET)
-        format.fmt.win.w.left = v4l2sink->overlay.left;
-      if (v4l2sink->overlay_fields_set & OVERLAY_WIDTH_SET)
-        format.fmt.win.w.width = v4l2sink->overlay.width;
-      if (v4l2sink->overlay_fields_set & OVERLAY_HEIGHT_SET)
-        format.fmt.win.w.height = v4l2sink->overlay.height;
+    if (v4l2sink->overlay_fields_set & OVERLAY_TOP_SET)
+      format.fmt.win.w.top = v4l2sink->overlay.top;
+    if (v4l2sink->overlay_fields_set & OVERLAY_LEFT_SET)
+      format.fmt.win.w.left = v4l2sink->overlay.left;
+    if (v4l2sink->overlay_fields_set & OVERLAY_WIDTH_SET)
+      format.fmt.win.w.width = v4l2sink->overlay.width;
+    if (v4l2sink->overlay_fields_set & OVERLAY_HEIGHT_SET)
+      format.fmt.win.w.height = v4l2sink->overlay.height;
 
-      g_return_if_fail (v4l2_ioctl (fd, VIDIOC_S_FMT, &format) >= 0);
-      v4l2sink->overlay_fields_set = 0;
-    }
+    g_return_if_fail (v4l2_ioctl (fd, VIDIOC_S_FMT, &format) >= 0);
+    v4l2sink->overlay_fields_set = 0;
 
     v4l2sink->overlay = format.fmt.win.w;
   }
@@ -466,6 +462,9 @@ gst_v4l2sink_change_state (GstElement * element, GstStateChange transition)
       }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
+      if (NULL != v4l2sink->pool)
+        gst_v4l2_buffer_pool_destroy (v4l2sink->pool);
+      v4l2sink->pool = NULL;
       /* close the device */
       if (!gst_v4l2_object_stop (v4l2sink->v4l2object))
         return GST_STATE_CHANGE_FAILURE;
@@ -598,7 +597,7 @@ gst_v4l2sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
   return TRUE;
 }
 
-/** buffer alloc function to implement pad_alloc for upstream element */
+/* buffer alloc function to implement pad_alloc for upstream element */
 static GstFlowReturn
 gst_v4l2sink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
     GstCaps * caps, GstBuffer ** buf)
@@ -651,7 +650,7 @@ gst_v4l2sink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
   }
 }
 
-/** called after A/V sync to render frame */
+/* called after A/V sync to render frame */
 static GstFlowReturn
 gst_v4l2sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 {
